<?php
namespace raas\apps\account;

// _group_count_inc load
ni_app_load('counter', '_lib');
ni_app_load('entowner', '_lib');
ni_app_load('enterpise', '_lib');
ni_app_load('enterpise', 'info');
ni_app_load('group', '_lib');
ni_app_load('group', 'info');
ni_app_load('notification', 'create');
ni_app_load('notification_to', 'create');


function _create($frmData)
{
	global $nidb;
	
	$frmData['update_time'] = time();
	if(isset($frmData['create_time']) == false)
		$frmData['create_time'] = $frmData['update_time'];

	try {
		if(insert_data('cloud_account', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$frmData['acct_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(\raas\apps\group\_acct_count_inc($frmData['group_id'], 1) === false){
		return false;
	}
	if(\raas\apps\enterpise\_acct_count_inc($frmData['ent_id'], 1) === false){
		return false;
	}
	if(\raas\apps\entowner\_acct_count_inc($frmData['own_id'], 1) === false){
		return false;
	}
	if(\raas\apps\counter\_acct_count_inc(COUNTER_RAAS_ID, 1) === false) {
		return false;
	}
	
	return $frmData;
}

function create($data)
{
	global $nidb, $user;

	$filter = array(
		'own_id'	=> new \FilterVarID(true),
		'group_id'	=> new \FilterVarID(true),
		'parent_id'	=> new \FilterVarInt(),
		'account'	=> new \FilterVarStr(true),
		'pass'		=> new \FilterVarStr(true),
		'disabled'	=> new \FilterVarInt(),
		'webuser_id'=> new \FilterVarInt(),
		'pxyacct'	=> new \FilterVarStr(),
		'pxypass'	=> new \FilterVarStr(),
		'pxyvlan_in'=> new \FilterVarInt(),
		'pxyvlan_out'=> new \FilterVarInt(),
		'create_time'=> new \FilterVarInt(),
		'start_time'=> new \FilterVarInt(),
		'expire_time'=> new \FilterVarInt(),
		'bind_ipv4'	=> new \FilterVarInt(),
		'bind_v4mac'=> new \FilterVarStr(),
		'bind_vlan'	=> new \FilterVarInt(),
		'rate_out'	=> new \FilterVarInt(),
		'rate_in'	=> new \FilterVarInt(),
		'maxcnt'	=> new \FilterVarInt(),
		'name'		=> new \FilterVarStr(),
		'idcard'	=> new \FilterVarStr(),
		'phone'		=> new \FilterVarStr(),
		'remark'	=> new \FilterVarStr(),
		'email'		=> new \FilterVarStr(),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(empty($frmData['account'])) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不能为空！');
		return false;
	}

	if(preg_match("/^([a-zA-Z0-9-_.]{2,})$/", $frmData['account'], $match) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号格式不正确！格式应为：a-z，A-Z，0-9和“.”，“_”，至少2位字符组成。');
		return false;
	}


	if(preg_match("/^([0-9]{2,})$/", $frmData['pass'], $match)) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '密码不能为纯数字。');
		return false;
	}
	if(preg_match("/^([a-z]{2,})$/", $frmData['pass'], $match)) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '密码不能为纯小写字母。');
		return false;
	}
	if(preg_match("/^([A-Z]{2,})$/", $frmData['pass'], $match)) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '密码不能为纯大写字母。');
		return false;
	}

	if(isset($data['asacctpassw']) && isset($frmData['pxyacct']) 
	&& empty($frmData['pxyacct']) == false
	&& intval($data['asacctpassw'])) {
		if(isset($frmData['pass'])) 
			$frmData['pxypass'] = $frmData['pass'];
	}
	
	// find group
	$info = array (
		'group_id'	=> $frmData['group_id'],
		'own_id'	=> $frmData['own_id'],
	);
	if(($grp = \raas\apps\group\info($info)) === false){
		set_errmsg(MSG_LEVEL_DEF, __function__, '请选择一个有效的用户组。');
		return false;
	}
	
	if(($ent = \raas\apps\enterpise\_info($grp->ent_id, 0, 1, 0)) === false){
		set_errmsg(MSG_LEVEL_DEF, __function__, '该用户组的归属项目不存在。');
		return false;
	}

	// 检查操作员创建帐号时，是否需要负责人审核
	if(gettype($user) == 'object' && $ent->admin != $user->account && $user->entcnt > 0) {
		$sql = "select * from cloud_user_group_list where `ent_id` = ?";
		$sql.= " and `user_id` = ? limit 1";

		try {
			$sth = $nidb->prepare($sql);

			$sth->bindParam(1, $grp->ent_id, \PDO::PARAM_INT);
			$sth->bindParam(2, $user->user_id, \PDO::PARAM_INT);

			if($sth->execute())
				$row = $sth->fetch(\PDO::FETCH_ASSOC);
			else
				$row = false;
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if(!$row) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '无法完成此操作，您的权限不足。');
			return false;
		}
		
		return create_plus($data);
	}

	if(isset($data['check_pxy_acct']) && intval($data['check_pxy_acct'])
	&& isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
		$sql = "select `account` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
		$sql.=" and `own_id` = ? and `ent_id` = ?";
		$sql.=" and `pxyacct` = ? limit 1";

		try {
			$sth = $nidb->prepare($sql);

			$index = 1;

			$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
			$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if($rows && count($rows) > 0) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '代拨帐号已存在。');
			return 'PXY_ACCT_EXIST';
		}
	}

	// find account is exist
	$sql = "select `group_id` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql .=" and `ent_id` = ? and `account` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['account'], \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) > 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, (empty($frmData['name'])?'':$frmData['name'] . '的') . '“' . $frmData['account'] . '”帐号已存在');
		return false;
	}
	
	$frmData['own_id'] = $grp->own_id;
	$frmData['ent_id'] = $grp->ent_id;
	
	$frmData['group'] = $grp->name;

	$nidb->beginTransaction();

	if(($frmData = _create($frmData)) === false) {
		$nidb->rollBack();
		return false;
	}
	
	// 修正所有代拨密码
	$rows = false;
	if(isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
		try {
			$where = array('pxyacct' => $frmData['pxyacct']);
			if(update_data('cloud_account', array('pxypass' => $frmData['pxypass']), $where) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				$nidb->rollBack();
				return false;
			}
			
			$sql = "select * from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
			$sql.=" and `own_id` = ? and `ent_id` = ?";
			$sql.=" and `pxyacct` = ? and `acct_id` != ?";

			$sth = $nidb->prepare($sql);

			$index = 1;

			$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
			$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);
			$sth->bindParam($index++, $frmData['acct_id'], \PDO::PARAM_INT);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			$nidb->rollBack();
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

	}

	$nidb->commit();
//	synclog_push(RA_SYNC_ACCOUNT, $frmData['acct_id']);
	raaseye_acct_add($frmData);

	if($rows) {
		foreach($rows as $row) {
			 raaseye_acct_add((array)$row);
		}
	}

	return $frmData;
}

function create_plus($data)
{
	global $nidb, $user;

	$filter = array(
		'own_id'	=> new \FilterVarID(true),
		'group_id'	=> new \FilterVarID(true),
		'parent_id'	=> new \FilterVarInt(),
		'account'	=> new \FilterVarStr(true),
		'pass'		=> new \FilterVarStr(true),
		'disabled'	=> new \FilterVarInt(),
		'webuser_id'=> new \FilterVarInt(),
		'pxyacct'	=> new \FilterVarStr(),
		'pxypass'	=> new \FilterVarStr(),
		'pxyvlan_in'=> new \FilterVarInt(),
		'pxyvlan_out'=> new \FilterVarInt(),
		'create_time'=> new \FilterVarInt(),
		'start_time'=> new \FilterVarInt(),
		'expire_time'=> new \FilterVarInt(),
		'bind_ipv4'	=> new \FilterVarInt(),
		'bind_v4mac'=> new \FilterVarStr(),
		'bind_vlan'	=> new \FilterVarInt(),
		'rate_out'	=> new \FilterVarInt(),
		'rate_in'	=> new \FilterVarInt(),
		'maxcnt'	=> new \FilterVarInt(),
		'name'		=> new \FilterVarStr(),
		'idcard'	=> new \FilterVarStr(),
		'phone'		=> new \FilterVarStr(),
		'remark'	=> new \FilterVarStr(),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(empty($frmData['account'])) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不能为空！');
		return false;
	}

	if(isset($data['asacctpassw']) && isset($frmData['pxyacct']) 
	&& empty($frmData['pxyacct']) == false
	&& intval($data['asacctpassw'])) {
		if(isset($frmData['pass'])) 
			$frmData['pxypass'] = $frmData['pass'];
	}
	
	/*
	if(strpos($frmData['account'], '@') !== false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不能有“@”字符！');
		return false;
	}
	*/
	if(preg_match("/^([a-zA-Z0-9-_.]{2,})$/", $frmData['account'], $match) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号格式不正确！格式应为：a-z，A-Z，0-9和“.”，“_”，至少2位字符组成。');
		return false;
	}

	// find group
	$info = array (
		'group_id'	=> $frmData['group_id'],
		'own_id'	=> $frmData['own_id'],
	);
	if(($grp = \raas\apps\group\info($info)) === false){
		set_errmsg(MSG_LEVEL_DEF, __function__, '请选择一个有效的用户组。');
		return false;
	}
	
	if(isset($data['check_pxy_acct']) && intval($data['check_pxy_acct'])
	&& isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
		$sql = "select `account` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
		$sql.=" and `own_id` = ? and `ent_id` = ?";
		$sql.=" and `pxyacct` = ? limit 1";

		try {
			$sth = $nidb->prepare($sql);

			$index = 1;

			$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
			$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if($rows && count($rows) > 0) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '代拨帐号已存在。');
			return 'PXY_ACCT_EXIST';
		}
	}

	// find account is exist
	$sql = "select `group_id` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql .=" and `ent_id` = ? and `account` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['account'], \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) > 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, (empty($frmData['name'])?'':$frmData['name'] . '的') . '“' . $frmData['account'] . '”帐号已存在');
		return false;
	}
	
	$frmData['own_id'] = $grp->own_id;
	$frmData['ent_id'] = $grp->ent_id;
	
	$frmData['group'] = $grp->name;

	// set io api
	$frmData['init'] = 'function (){
	}';
	$json_data = json_encode($frmData);
	

	$iwan = array();

	$iwan['update_time'] = time();


	$iwan['own_id']		= $user->own_id;
	$iwan['ent_id'] 	= $grp->ent_id;
	$iwan['user_id'] 	= $user->user_id;
	$iwan['creator']	= $user->name;


	$iwan['method']	  = 'account.create';
	
	$iwan['name']	  = $frmData['name'];
	$iwan['mask']	  = 7; // 1：打开阅读计数，2：同意，4：拒绝
	$iwan['message']  = '申请开通认证帐号';

	$iwan['data']	  = $json_data;
	$iwan['content']  = '<p>' . $frmData['remark'] . '</p>';
	$iwan['content'] .= '<p>姓名：' . $frmData['name'];
	$iwan['content'] .= '<br>帐号：' . $frmData['account'];
	$iwan['content'] .= '<br>密码：' . $frmData['pass'];

	$iwan['content'] .= '<br>身份证：';
	if(isset($frmData['idcard']))
		$iwan['content'] .= $frmData['idcard'];
	
	$iwan['content'] .= '<br>联系电话：';
	if(isset($frmData['phone']))
		$iwan['content'] .= $frmData['phone'];

	$iwan['content'] .= '<br>起始使用时间：';
	if(isset($frmData['start_time']))
		$iwan['content'] .= date('Y-m-d H:i:s', $frmData['start_time']);

	$iwan['content'] .= '<br>截止使用时间：';
	if(isset($frmData['expire_time']))
		$iwan['content'] .= date('Y-m-d H:i:s', $frmData['expire_time']);

	
	$iwan['content'] .= '</p>';



	$iwan['content'] .= '<p>绑定IP：';
	if(isset($frmData['bind_ipv4']) && $frmData['bind_ipv4'] != 0)
		$iwan['content'] .= long2ip($frmData['bind_ipv4']);

	$iwan['content'] .= '<br>绑定MAC：';
	if(isset($frmData['bind_v4mac']))
		$iwan['content'] .= $frmData['bind_v4mac'];

	$iwan['content'] .= '<br>绑定VLAN：';
	if(isset($frmData['bind_vlan']))
		$iwan['content'] .= $frmData['bind_vlan'];
	
	$iwan['content'] .= '<br>最大在线数：';
	if(isset($frmData['maxcnt']))
		$iwan['content'] .= $frmData['maxcnt'];
	
	$iwan['content'] .= '<br>代拨账号：';
	if(isset($frmData['pxyacct']))
		$iwan['content'] .= $frmData['pxyacct'];
	
	$iwan['content'] .= '<br>代拨密码：';
	if(isset($frmData['pxypass']))
		$iwan['content'] .= $frmData['pxypass'];

	$iwan['content'] .= '</p>';

	$iwan['content'] .= '<p>操作员：' . $user->name . ' [' . $user->account . ']';
	$iwan['content'] .= '<br>操作IP地址：' . get_real_ip() . '</p>';

	$nidb->beginTransaction();

	try {
		if(($notification = \raas\apps\notification\_create($iwan)) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}

		$notification['own_id']		= $grp->own_id;
		$notification['ent_id']		= $grp->ent_id;
		$notification['user_id']	= 0;

		unset($notification['data']);
		unset($notification['content']);
		unset($notification['method']);

		if(($notification_to = \raas\apps\notification_to\_create($notification)) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();

	$frmData['notification'] = 1; 
	return $frmData;
}