<?php
namespace raas\apps\account;

// 支持修改分组
// _acct_count_inc load
ni_app_load('enterpise', '_lib');
ni_app_load('enterpise', 'info');
ni_app_load('group', '_lib');
ni_app_load('group', 'info');
ni_app_load('account', 'find');
ni_app_load('account', 'info');

function _save_groupname($group_id, $name)
{
	global $nidb;

	$sql = "update cloud_account
			set `group` = ?
			where `group_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		
		$index = 1;
		$sth->bindParam($index++, $name, \PDO::PARAM_STR);
		$sth->bindParam($index++, $group_id, \PDO::PARAM_INT);
		
		$sth->execute();
		return $sth->rowCount();
	}
	catch (PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	return 0;
}

function save($data)
{
	global $nidb;

	$filter = array(
		'acct_id'	=> new \FilterVarID(),
		'account'	=> new \FilterVarStr(),
		'ent_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(isset($where['acct_id']) == false
	&& isset($where['account']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '帐号不能为空。');
		return false;
	}

	$filter = array(
		'group_id'	=> new \FilterVarID(),
//		'iwan_id'	=> new \FilterVarInt(),
//		'group'		=> new \FilterVarStr(),
		'pass'		=> new \FilterVarStr(),
		'disabled'	=> new \FilterVarInt(),
		'webuser_id'=> new \FilterVarInt(),
		'pxyacct'	=> new \FilterVarStr(),
		'iwanacct'	=> new \FilterVarStr(),
		'pxypass'	=> new \FilterVarStr(),
		'pxyvlan_in'=> new \FilterVarInt(),
		'pxyvlan_out'=> new \FilterVarInt(),
		'start_time'=> new \FilterVarInt(),
		'expire_time'=> new \FilterVarInt(),
		'bind_ipv4'	=> new \FilterVarInt(),
		'bind_v4mac'=> new \FilterVarStr(),
		'bind_vlan'	=> new \FilterVarInt(),
		'rate_out'	=> new \FilterVarInt(),
		'rate_in'	=> new \FilterVarInt(),
		'maxcnt'	=> new \FilterVarInt(),
		'name'		=> new \FilterVarStr(),
		'idcard'	=> new \FilterVarStr(),
		'phone'		=> new \FilterVarStr(),
		'remark'	=> new \FilterVarStr(),
		'email'		=> new \FilterVarStr(),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	if(count($frmData) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "修改已放弃！无任何内容需要保存。");
		return false;
	}

	if(isset($data['pass'])) {
		if(preg_match("/^([0-9]{2,})$/", $data['pass'], $match)) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '密码不能为纯数字。');
			return false;
		}
		if(preg_match("/^([a-z]{2,})$/", $data['pass'], $match)) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '密码不能为纯小写字母。');
			return false;
		}
		if(preg_match("/^([A-Z]{2,})$/", $data['pass'], $match)) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '密码不能为纯大写字母。');
			return false;
		}
	}

	if(isset($data['check_pxy_acct']) && intval($data['check_pxy_acct'])
	&& isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
		$sql = "select `account` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
		$sql.=" and `own_id` = ? and `ent_id` = ?";
		$sql.=" and `pxyacct` = ? ";
		
		if(isset($where['acct_id']))
			$sql .=" and `acct_id` != ?";
		if(isset($where['account']))
			$sql .=" and `account` != ?";

		$sql.=" limit 1";

		try {
			$sth = $nidb->prepare($sql);

			$index = 1;

			$sth->bindParam($index++, $where['own_id'], \PDO::PARAM_INT);
			$sth->bindParam($index++, $where['ent_id'], \PDO::PARAM_INT);
			$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);

			if(isset($where['acct_id']))
				$sth->bindParam($index++, $where['acct_id'], \PDO::PARAM_INT);
			if(isset($where['account']))
				$sth->bindParam($index++, $where['account'], \PDO::PARAM_STR);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
		
		if($rows && count($rows) > 0) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '代拨帐号已存在。');
			return 'PXY_ACCT_EXIST';
		}
	}

	$frmData['update_time'] = time();
	$ent_id = 0;

	// find group_id of account
	$sql = "select `acct_id`, `account`, `pass`, `group_id`, `ent_id`, `own_id` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql .=" and `own_id` = ? and `ent_id` = ?";
	
	
	if(isset($where['acct_id']))
		$sql .=" and `acct_id` = ?";
	if(isset($where['account']))
		$sql .=" and `account` = ?";

	try {
		$sth = $nidb->prepare($sql);

		$index = 1;

		$sth->bindParam($index++, $where['own_id'], \PDO::PARAM_INT);
		$sth->bindParam($index++, $where['ent_id'], \PDO::PARAM_INT);

		if(isset($where['acct_id']))
			$sth->bindParam($index++, $where['acct_id'], \PDO::PARAM_INT);
		if(isset($where['account']))
			$sth->bindParam($index++, $where['account'], \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '该用户不存在！');
		return false;
	}
	$row = $rows[0];

	if(isset($data['asacctpassw']) && isset($frmData['pxyacct'])
	&& empty($frmData['pxyacct']) == false
	&& intval($data['asacctpassw'])) {
		if(isset($frmData['pass'])) 
			$frmData['pxypass'] = $frmData['pass'];
		else
			$frmData['pxypass'] = $row->pass;
	}

	// is change group_id ?
	if(isset($frmData['group_id'])) {
		$info = array (
			'group_id' => $frmData['group_id'],
			'own_id' => $where['own_id'],
		);
		if(($grp = \raas\apps\group\info($info)) === false){
			set_errmsg(MSG_LEVEL_DEF, __function__, '请选择一个有效的用户组。');
			return false;
		}
		$frmData['group'] = $grp->name;
		
		if($row->ent_id != $grp->ent_id) {
			$usr = _find($row->account, $grp->ent_id, $row->own_id);
			if($usr !== false)  {
				set_errmsg(MSG_LEVEL_DEF, __function__, '相同的用户已存在！' . $row->ent_id . ' != ' . $grp->ent_id);
				return false;
			}
			set_errmsg(MSG_LEVEL_DEF, __function__, '');
		}

		$nidb->beginTransaction();
		// reset count of group
		if($row->group_id != $grp->group_id) {
			// old group del acctcnt
			if($row->group_id > 0) {
				if(\raas\apps\group\_acct_count_inc($row->group_id, -1) === false) {
					$nidb->rollBack();
					set_errmsg(MSG_LEVEL_DEF, __function__, $row->group_id . '用户组计数失败！');
					return false;
				}
			}
			// new group add acctcnt
			if($grp->group_id > 0) {
				if(\raas\apps\group\_acct_count_inc($grp->group_id, 1) === false) {
					$nidb->rollBack();
					set_errmsg(MSG_LEVEL_DEF, __function__, $grp->group_id . '用户组计数失败！');
					return false;
				}
			}
			
			// if ent_id nq then
			if($row->ent_id != $grp->ent_id) {
				// old ent del acctcnt
				if($row->ent_id > 0) {
					$ent_id = $row->ent_id;
					if(\raas\apps\enterpise\_acct_count_inc($row->ent_id, -1) === false) {
						$nidb->rollBack();
						set_errmsg(MSG_LEVEL_DEF, __function__, '项目用户组' . $grp->group_id . '计数失败！');
						return false;
					}
				}
				// new ent add acctcnt
				if($grp->ent_id > 0) {
					if(\raas\apps\enterpise\_acct_count_inc($grp->ent_id, 1) === false) {
						$nidb->rollBack();
						set_errmsg(MSG_LEVEL_DEF, __function__, '项目用户组' . $grp->group_id . '计数失败！');
						return false;
					}
				}

				// reset ent_id
				$frmData['ent_id'] = $grp->ent_id;
			}
		}
	}
	else
		$nidb->beginTransaction();

	try {
		if(update_data('cloud_account', $frmData, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
	}
	catch (PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}
	
	// 修正所有代拨密码
	$rows = false;
	if(isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
		try {
			$whereX = array('pxyacct' => $frmData['pxyacct']);
			if(update_data('cloud_account', array('pxypass' => $frmData['pxypass']), $whereX) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				$nidb->rollBack();
				return false;
			}
			
			$sql = "select * from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
			$sql.=" and `own_id` = ? and `ent_id` = ?";
			$sql.=" and `pxyacct` = ? and `acct_id` != ?";

			$sth = $nidb->prepare($sql);

			$index = 1;

			$sth->bindParam($index++, $grp->own_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);
			$sth->bindParam($index++, $row->acct_id, \PDO::PARAM_INT);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			$nidb->rollBack();
			return false;
		}

	}

	$nidb->commit();

//	synclog_push(RA_SYNC_ACCOUNT, $where['acct_id']);
	if(isset($frmData['ent_id']))
		$where['ent_id'] = $frmData['ent_id'];
	$info = info($where, 1);
	raaseye_acct_add($info);

	if($ent_id > 0) {
		$ent = \raas\apps\enterpise\_info($ent_id);
		raaseye_acct_remove($row->account . '@' . $ent->domain);
	}

	if($rows) {
		foreach($rows as $row) {
			 raaseye_acct_add((array)$row);
		}
	}

	return true;
}

