<?php
namespace raas\apps\bindmac;


function create($data)
{
	global $nidb, $user;

	$filter = array(
		'own_id'	=> new \FilterVarID(true),
		'acct_id'	=> new \FilterVarID(true),
		'mac'		=> new \FilterVarStr(),
		'name'		=> new \FilterVarStr(),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	
	if(preg_match("/^[A-Fa-f0-9]{1,2}\:[A-Fa-f0-9]{1,2}\:[A-Fa-f0-9]{1,2}\:[A-Fa-f0-9]{1,2}\:[A-Fa-f0-9]{1,2}\:[A-Fa-f0-9]{1,2}$/", $frmData['mac'], $match) == false
	&& preg_match("/^[A-Fa-f0-9]{1,2}\-[A-Fa-f0-9]{1,2}\-[A-Fa-f0-9]{1,2}\-[A-Fa-f0-9]{1,2}\-[A-Fa-f0-9]{1,2}\-[A-Fa-f0-9]{1,2}$/", $frmData['mac'], $match) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'MAC地址格式无效。');
		return false;
	}

	$sql = "select `account` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql.=" and `own_id` = ? and `acct_id` = ? limit 1";

	try {
		$sth = $nidb->prepare($sql);

		$index = 1;

		$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['acct_id'], \PDO::PARAM_INT);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$rows || count($rows) <= 0) {
		set_errmsg(MSG_LEVEL_EXP, __function__, '无效的帐号。');
		return false;
	}

	// find mac is exist
	$sql = "select `disabled` from cloud_account_bindmac where `acct_id` = ? and `mac` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $frmData['acct_id'], \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['mac'], \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0) {
		if($rows[0]->disabled < TB_ROW_DEL_VALUE) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '“' . $frmData['mac'] . '”设备已存在');
			return false;
		}
		$where = array(
			'own_id'	=> $frmData['own_id'],
			'acct_id'	=> $frmData['acct_id'],
			'mac'		=> $frmData['mac'],
		);
		$upData = array(
			'disabled' => 0,
			'update_time' => time(),
		);
		$upData['create_time'] = $upData['update_time'];
		try {
			if(update_data('cloud_account_bindmac', $upData, $where) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				return false;
			}
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}
	else {
		$frmData['update_time'] = time();
		if(isset($frmData['create_time']) == false)
			$frmData['create_time'] = $frmData['update_time'];

		try {
			if(insert_data('cloud_account_bindmac', $frmData) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				return false;
			}
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}

	raaseye_acct_bindmac_reload(0);

	return $frmData;
}
