<?php
namespace raas\apps\enterpise;

ni_app_load('group', 'create');
ni_app_load('group', 'find');
ni_app_load('group', 'info');
ni_app_load('enterpise', 'info');
ni_app_load('account', 'create');
ni_app_load('account', 'save');

// function
function _getpagname($name, $ent_id)
{
	global $nidb, $user;

	if(empty($name)) return false;
	
	// set custom options
	$where_str = "where `disabled` < " . TB_ROW_DEL_VALUE . " and `name` = ?";
	$where_str .= " and `ent_id` = ? and `own_id` = ?";

	$sql = "select * from cloud_group $where_str";

	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$own_id = $user->own_id;
		$sth->bindParam($index++, $name, \PDO::PARAM_STR);
		$sth->bindParam($index++, $ent_id, \PDO::PARAM_INT);
		$sth->bindParam($index++, $own_id, \PDO::PARAM_INT);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "无效的分组。");
		return false;
	}
	
	return $rows[0];
}

function _import_account_from_panabit($acctfile, $ent_id, $group_id, $charset)
{
	global $user;
	
	$result = array(
		'groupcnt'	=> 0,
		'acctcnt'	=> 0,
		'acctmax'	=> 0,
		'error'		=> array(),
		'err'		=> '',
	);
	
	if(\raas\apps\enterpise\info(array('ent_id' => $ent_id, 'own_id' => $user->own_id)) === false) {
		array_push($result['error'], '找不到指定要导入的项目。');
		return $result;
	}
	
	$importgrp = array();
	if($group_id > 0) {
		if(($grp = \raas\apps\group\info(array('ent_id' => $ent_id, 'group_id' => $group_id))) == false) {
			array_push($result['error'], '无效的用户组。');
			return $result;
		}
		if($grp->own_id != $user->own_id) {
			array_push($result['error'], '找不到该用户组。');
			return $result;
		}
	}
	
	$fd = fopen($acctfile["tmp_name"], "r");
	if ($fd) {

		while (($buff = fgets($fd, 4096)) !== false) {
			$lines = explode(' ', str_replace(PHP_EOL, '', $buff));
			if($group_id <= 0 && $lines[0] == 'g') {
				array_shift($lines);
				$map_grp = array(
					'id'			=> 'pagid',
					'name'			=> 'name', 
					'start'			=> 'startip', 
					'end'			=> 'endip', 
					'ratein'		=> 'rate_in', 
					'rateout'		=> 'rate_out', 
					'maxonlinetime'	=> 'maxonlinetm', 
					'ifname'		=> 'ifname'
				);
				$ret = array();
/*
  ["id"]=>
  string(1) "2"
  ["name"]=>
  string(3) "AAA"
  ["start"]=>
  string(10) "10.10.10.2"
  ["end"]=>
  string(12) "10.10.10.255"
  ["ratein"]=>
  string(3) "222"
  ["rateout"]=>
  string(4) "1111"
  ["dns"]=>
  string(15) "114.114.114.114"
  ["maxonlinetime"]=>
  string(3) "500"
  ["clntepa"]=>
  string(4) "pass"
  ["ifname"]=>
  string(3) "le1"
*/
				foreach($lines as $volume)
				{
					$node = explode('=', $volume);
					if(count($node) < 2) 
						continue;

					if(isset($map_grp[$node[0]])) {
						if($node[0] == 'start' || $node[0] == 'end') 
							$ret[$map_grp[$node[0]]] =  ip2long($node[1]);
						else
						if($node[0] == 'name' && $charset == 'gb2312')
							$ret[$map_grp[$node[0]]] =  iconv('GB2312', 'UTF-8', $node[1]);
						else
							$ret[$map_grp[$node[0]]] =  $node[1];
					}
					else 
					if($node[0] == 'dns') {
						$dns = explode(',', $node[1]);
						if(count($dns) > 0) $ret['dns0'] = ip2long($dns[0]);
						if(count($dns) > 1) $ret['dns1'] = ip2long($dns[1]);
					}
					else 
					if($node[0] == 'clntepa') {
						$clntepa = array('reject', 'login', 'pass');
						if(($key = array_search($node[1], $clntepa)) !== NULL)
							$ret[$node[0]] =  $key;
					}
				}
				$ret['ent_id'] = $ent_id;
				$ret['own_id'] = $user->own_id;
				if(($grp = \raas\apps\group\create($ret)) !== false) {
					$result['groupcnt']++;
				}
				else 
					array_push($result['error'], get_errmsg(''));

				$importgrp[$ret['pagid']] = $ret['name'];
			}
			else
			if($lines[0] == 'u') {
				array_shift($lines);
				$map_grp = array (
					'n'			=> 'account', 
					'p'			=> 'pass',
					'b'			=> 'birth_time', 
					's'			=> 'start_time', 
					'e'			=> 'expire_time', 
					'ip'		=> 'bind_ipv4', 
					'mac'		=> 'bind_v4mac', 
					'maxonline'	=> 'maxcnt', 
					'outvlan'	=> 'bind_vlan',
				);
				$ret = array();
				$pagid = 0;
/*
  ["g"]=>
  string(1) "1"
  ["n"]=>
  string(2) "aa"
  ["p"]=>
  string(1) "1"
  ["b"]=>
  string(10) "1529645858"
  ["e"]=>
  string(10) "2019-06-22"
  ["s"]=>
  string(10) "2018-06-22"
  ["mac"]=>
  string(17) "10-00-00-00-00-10"
  ["maxonline"]=>
  string(2) "10"
  ["ip"]=>
  string(9) "11.0.0.10"
  ["outvlan"]=>
  string(2) "33"
  ["m"]=>
  string(24) "0;eee;wef;18301045608;VV"
*/
				foreach($lines as $volume)
				{
					$node = explode('=', $volume);
					if(count($node) < 2) 
						continue;
					
					if(isset($map_grp[$node[0]])) {
						if($node[0] == 's' || $node[0] == 'e') {
							$udate = new \DateTime($node[1]);
							$ret[$map_grp[$node[0]]] =  $udate->getTimestamp();
						}
						else
						if($node[0] == 'ip')
							$ret[$map_grp[$node[0]]] =  ip2long($node[1]);
						//if($node[0] == 'mac')
						//	$ret[$map_grp[$node[0]]] =  str_replace(':', '-', $node[1]);
						else
							$ret[$map_grp[$node[0]]] =  $node[1];
					} 
					else 
					if($node[0] == 'm') {
						if($charset == 'gb2312')
							$node[1] = iconv('GB2312', 'UTF-8', $node[1]);
							
						$m = explode(';', $node[1]);
						if(count($m) > 1) $ret['name'] =  $m[1];
						if(count($m) > 2) $ret['idcard'] =  $m[2];
						if(count($m) > 3) $ret['phone'] =  $m[3];
						if(count($m) > 4) $ret['remark'] =  $m[4];
					}
					else 
					if($node[0] == 'g') {
						$pagid = $node[1];
					}
				}
				$ret['ent_id'] = $ent_id;
				$ret['own_id'] = $user->own_id;
				if($group_id > 0) {
					$ret['group_id'] = $group_id;
				}
				else {
					if(isset($importgrp[$pagid]) == false
					||($grp = _getpagname($importgrp[$pagid], $ent_id)) === false)
						continue;
					$ret['group_id'] = $grp->group_id;
				}
				$result['acctmax']++;

				if(\raas\apps\account\create($ret) !== false)
					$result['acctcnt']++;
				else 
				if(strpos(get_errmsg(), '已存在') !== false){
					if(\raas\apps\account\save($ret) !== false)
						$result['acctcnt']++;
					else
						array_push($result['error'], get_errmsg(''));
				}
				else
					array_push($result['error'], get_errmsg(''));
			}
		}
	}
	fclose($fd);
	
	if(count($result['error']) <= 0)
		set_errmsg(MSG_LEVEL_DEF, __function__, '');
	
	return $result;
}

function _import_account_from_csv($acctfile, $ent_id, $group_id, $charset)
{
	global $user;
	
	$length = 8096;
	$delimiter = ',';
	$enclosure = '"';
	$escape = '\\';

	
	$result = array(
		'groupcnt'	=> 0,
		'acctcnt'	=> 0,
		'acctmax'	=> 0,
		'error'		=> array(),
		'err'		=> '',
	);

	if(\raas\apps\enterpise\info(array('ent_id' => $ent_id, 'own_id' => $user->own_id)) === false) {
		array_push($result['error'], '找不到指定要导入的项目。');
		return $result;
	}

	if($group_id > 0) {
		if(($grp = \raas\apps\group\info(array('ent_id' => $ent_id, 'group_id' => $group_id))) == false) {
			array_push($result['error'], '无效的用户组。');
			return $result;
		}
		if($grp->own_id != $user->own_id) {
			array_push($result['error'], '找不到该用户组。');
			return $result;
		}
	}
	
	$handle = fopen($acctfile, 'r');
	if (!$handle) {
		array_push($result['error'], '打开文件失败！');
		return $result;
	}
	$data = array();
	$bom =( chr(0xEF) . chr(0xBB) . chr(0xBF) );
	$group = array();
	$volume = array(
		'account', 
		'pass',
		'create_time', 
		'start_time', 
		'expire_time',
		'group_id',
		'bind_ipv4', 
		'bind_v4mac',
		'bind_vlan',
		'maxcnt', 
		'pxyacct',
		'pxypass',
		'pxyvlan_in',
		'pxyvlan_out',
		'name',
		'remark',
		'idcard',
		'phone',
	);
	$keycnt = count($volume);
	$errnum_start = 0;
	$errnum_next = 0;
	
	$linecnt= 0;
	if(($line = fgetcsv($handle, $length, $delimiter, $enclosure)) !== false) {

		// 帐号，密码，创建时间，开始时间，到期时间，用户组，绑定IP，绑定MAC，绑定VLAN，最大在线数，
		// 代拨账号，代拨密码，代拨VLAN内，代拨VLAN外，姓名，备注，身份证，联系电话
		$line[0] = str_replace($bom, '', $line[0]);
		// $count = count($line);

		do {
			$linecnt++;
			if($keycnt == count($line)) {
				$str = trim($line[0]);
				if(empty($str) || $str[0] == '#') continue;

				$row = array_combine($volume, $line);

				if($charset == 'gb2312') {
					$row['group_id'] = iconv('GB2312', 'UTF-8', $row['group_id']);
					$row['account'] = iconv('GB2312', 'UTF-8', $row['account']);
					$row['pass']	= iconv('GB2312', 'UTF-8', $row['pass']);
					$row['pxyacct'] = iconv('GB2312', 'UTF-8', $row['pxyacct']);
					$row['name']	= iconv('GB2312', 'UTF-8', $row['name']);
					$row['remark']	= iconv('GB2312', 'UTF-8', $row['remark']);
					$row['idcard']	= iconv('GB2312', 'UTF-8', $row['idcard']);
					$row['phone']	= iconv('GB2312', 'UTF-8', $row['phone']);
				}
				if($group_id <= 0) {
					if(isset($group[$row['group_id']]) == false) {
						if(($grp = \raas\apps\group\find(array('ent_id' => $ent_id, 'name' => $row['group_id']))) == false) {
							array_push($result['error'], '无法找到第“' . $linecnt . '”行记的用户组“' . $row['group_id'] . '”。' . get_errmsg(''));
							break;
						}
						$group[$row['group_id']] = $grp->group_id;
					}

					$row['group_id'] =$group[$row['group_id']];
				}
				else {
					$row['group_id'] = $group_id;
				}

				//  birth_time start_time expire_time
				$date = new \DateTime($row['create_time']);
				$row['create_time'] = $date->getTimestamp();
				$date = new \DateTime($row['start_time']);
				$row['start_time'] = $date->getTimestamp();
				$date = new \DateTime($row['expire_time']);
				$row['expire_time'] = $date->getTimestamp();

				$row['own_id'] = $user->own_id;
				$row['ent_id'] = $ent_id;

				$result['acctmax']++;
				//array_push($data, $row);
				//if(count($data) >= 50) {
				//	$result['acctcnt'] += count($data);
					if(\raas\apps\account\create($row) !== false)
						$result['acctcnt']++;
					else
					if(strpos(get_errmsg(), '已存在') !== false && \raas\apps\account\save($row) !== false){
						$result['acctcnt']++;
					}
					else {
						$result['err'] = get_errmsg('');
						if($errnum_next != $linecnt) {
							if($errnum_start)
								array_push($result['error'], '第“' . $errnum_start . '”到“' . $errnum_next . '”行记录导入失败。');
							
							$errnum_start = $linecnt;
							$errnum_next = $linecnt + 1;
						}
					}

/*
					if(import_account($data) == false) {
						array_push($result['error'], "第“" . ($linecnt - count($data)) . "”到“$linecnt”行记录导入失败。");
						break;
					}
*/
				//	$data = array();
				//}
			}
			else 
			if(empty($line[0]) == false && $line[0][0] != '#') {
				array_push($result['error'], '第“' . $linecnt . '”行记录格式无效。');
				break;
			}
		} while (($line = fgetcsv($handle, $length, $delimiter, $enclosure)) !== false);

	}

	fclose($handle);

	if($errnum_start) {
		if($errnum_next > $linecnt && $errnum_start < $linecnt)
			array_push($result['error'], '第“' . $errnum_start . '”到“' . $errnum_next . '”行记录导入失败。');
		else
			array_push($result['error'], '第“' . $errnum_start . '”行记录导入失败。');
	}
/*	
	if(count($data)) {
		$result['acctcnt'] += count($data);
		if(import_account($data) == false) {
			array_push($result['error'], "第“" . ($linecnt - count($data)) . "”到“$linecnt”行记录导入失败。");
			break;
		}
	}
*/
	if(count($result['error']) <= 0)
		set_errmsg(MSG_LEVEL_DEF, __function__, '');

	return $result; 
}
