<?php
namespace raas\apps\entowner;

function save($data)
{
	global $nidb;

	$filter = array(
		'own_id'	=> new \FilterVarID(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	$filter = array(
		'name'		=> new \FilterVarStr(),
		'disabled'	=> new \FilterVarInt(),
		'account'	=> new \FilterVarStr(),
		'contact'	=> new \FilterVarStr(),
		'phone'		=> new \FilterVarStr(),
		'position_id'=> new \FilterVarID(),
		'province'	=> new \FilterVarStr(),
		'city'		=> new \FilterVarStr(),
		'district'	=> new \FilterVarStr(),
		'address'	=> new \FilterVarStr(),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	if(count($frmData) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "修改已放弃！无任何内容需要保存。");
		return false;
	}
	
	if(isset($frmData['name']) && empty($frmData['name'])) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '项目名称不能为空！');
		return false;
	}
	
	$frmData['update_time'] = time();

	// is change account ?
	if(isset($frmData['account'])) {
		if(($u = \raas\apps\user\_getacct($frmData['account'])) === false) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '不存在的管理帐号！');
			return false;
		}
		if($u->own_id != $where['own_id']) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '找不到此帐号！');
			return false;
		}

		// find account of entowner
		$sql = "select `account` from cloud_entowner where `disabled` < " . TB_ROW_DEL_VALUE;
		$sql .=" and `own_id` = ?";
		try {
			$sth = $nidb->prepare($sql);

			$index = 1;
			$sth->bindParam($index++, $where['own_id'], \PDO::PARAM_INT);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
		if(count($rows) < 0) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '无效的项目！');
			return false;
		}
		$row = $rows[0];
		
		$nidb->beginTransaction();
		if($row->account != $frmData['account']) {
			
			// remove old account webmanage tag
			if($row->account != '') {
				if(\raas\apps\user\_webmanage_del($u->account, WEB_TYPE_ROOT) === false) {
					$nidb->rollBack();
					return false;
				}
			}
			
			// set new account webmanage tag
			if($frmData['account'] != '') {
				if(\raas\apps\user\_webmanage_set($frmData['account'], WEB_TYPE_ROOT) === false) {
					$nidb->rollBack();
					return false;
				}
			}
		}
	}
	else
		$nidb->beginTransaction();

	try {
		if(update_data('cloud_entowner', $frmData, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}
	
	$nidb->commit();

	return true;
}

