<?php
namespace raas\apps\forgot;

// load lib
ni_library_load('email');

// _entreg_count_inc load
ni_app_load('counter', '_lib');
ni_app_load('entowner', '_lib');
ni_app_load('enterpise', '_lib');
ni_app_load('forgot', 'info');
ni_app_load('forgot', 'find');
ni_app_load('user', 'save');
ni_app_load('account', 'info');

function auth($data)
{
	global $nidb;

	$filter = array(
		'key'		=> new \FilterVarID(true, 'fgt_id'),
		'code'		=> new \FilterVarInt(true, 'code'),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	$update_time = time();

	if(($reguser = _info($frmData['fgt_id'])) === false) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '此帐号尚未登记！');
			return false;
	}
	
	// find last user
	if(($lastuser = _findlast($frmData['account'])) !== false) {
		if($lastuser->fgt_id != $reguser->fgt_id) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '此链接已过期！');
			return false;
		}
	}
	else 
		set_errmsg(MSG_LEVEL_DEF, __function__, '');

	if($reguser->update_time > 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '此链接已过期！');
		return false;
	}

	if(($sptm = ($update_time - $reguser->create_time)) > 1800) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '此链接已过期！请重新获取重置邮件！');
		return false;
	}

	if($reguser->code != $frmData['code']) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的重置链接！请重新获取重置邮件！');
		return false;
	}
	
	$nidb->beginTransaction();
	if($reguser->type) {
		// 1：认证帐号找回密码。
		$acct = \raas\apps\account\info(array(
			'acct_id'	=> $reguser->user_id,
			'own_id'	=> $reguser->own_id,
		), 1);
		
		if(!$acct) {
			set_errmsg(MSG_LEVEL_DEF, __function__, "无效的帐号，帐号可能已被禁用或冻结。");
			return false;
		}

		try { 
			// save account
			$sql = "update cloud_account set `update_time` = {$update_time}, `pass` = '{$reguser->pass}', `forgotcnt` = `forgotcnt` + 1";
			$sql.= " where `acct_id` = {$acct->acct_id} and `own_id` = {$reguser->own_id}";
			$sth = $nidb->prepare($sql);
			$sth->execute();
			
			$acct = (array)$acct;
			$acct['pass'] = $reguser->pass;

			// update cloud_forgot
			$sql = "update cloud_forgot set `update_time` = ? where `fgt_id` = ?";
			$sth = $nidb->prepare($sql);

			$index = 1;
			$sth->bindParam($index++, $update_time, \PDO::PARAM_INT);
			$sth->bindParam($index++, $reguser->fgt_id, \PDO::PARAM_INT);

			$sth->execute();
			$cnt = $sth->rowCount();

			if($cnt > 0) {
				if(\raas\apps\counter\_forgot2_count_inc(COUNTER_RAAS_ID, 1) === false) {
					$nidb->rollBack();
					return false;
				}
				if(\raas\apps\entowner\_forgot2_count_inc($reguser->own_id, 1) === false) {
					$nidb->rollBack();
					return false;
				}
				if($reguser->ent_id > 0 && \raas\apps\enterpise\_forgot_count_inc($reguser->ent_id, 1) === false) {
					$nidb->rollBack();
					return false;
				}
			}

		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			$nidb->rollBack();
			return false;
		}

	}
	else {
		// 0：操作员找回密码
		try { 
			// save user
			$sql = "update cloud_user set `update_time` = {$update_time}, `pass` = '{$reguser->pass}', `forgotcnt` = `forgotcnt` + 1";
			$sql.= " where `user_id` = {$reguser->user_id} and `own_id` = {$reguser->own_id}";
			$sth = $nidb->prepare($sql);
			$sth->execute();

			// update cloud_forgot
			$sql = "update cloud_forgot set `update_time` = ? where `fgt_id` = ?";
			$sth = $nidb->prepare($sql);

			$index = 1;
			$sth->bindParam($index++, $update_time, \PDO::PARAM_INT);
			$sth->bindParam($index++, $reguser->fgt_id, \PDO::PARAM_INT);

			$sth->execute();
			$cnt = $sth->rowCount();
			
			if($cnt > 0) {
				if(\raas\apps\counter\_forgot_count_inc(COUNTER_RAAS_ID, 1) === false) {
					$nidb->rollBack();
					return false;
				}
				if(\raas\apps\entowner\_forgot_count_inc($reguser->own_id, 1) === false) {
					$nidb->rollBack();
					return false;
				}
			}

		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			$nidb->rollBack();
			return false;
		}
	}

	if($cnt <= 0) {
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();
	
	// 如果是认证帐号，则更新信息到 paraa 服务
	if($reguser->type && $acct)
		raaseye_acct_add($acct);
	
	return $cnt;
}

