<?php
namespace raas\apps\group;

// _group_count_inc load
ni_app_load('counter', '_lib');
ni_app_load('entowner', '_lib');
ni_app_load('enterpise', '_lib');
ni_app_load('enterpise', 'info');

function create($data)
{
	global $nidb;

	$filter = array(
		'ent_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
		'name'		=> new \FilterVarStr(true),
		'pagid'		=> new \FilterVarID(),
		'disabled'	=> new \FilterVarInt(),
		'startip'	=> new \FilterVarInt(),
		'service'	=> new \FilterVarInt(),
		'endip'		=> new \FilterVarInt(),
		'dns0'		=> new \FilterVarInt(),
		'dns1'		=> new \FilterVarInt(),
		'rate_out'	=> new \FilterVarKBit(),
		'rate_in'	=> new \FilterVarKBit(),
		'maxonlinetm'=> new \FilterVarInt(),
		'clntepa'	=> new \FilterVarInt(),
		'ifname'	=> new \FilterVarStr(),
		'vlan_in'	=> new \FilterVarInt(),
		'vlan_out'	=> new \FilterVarInt(),
		'remark'	=> new \FilterVarStr(),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(empty($frmData['name'])) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'err: name can\'t empty.');
		return false;
	}
	
	// find enterpise obj
	$ent = \raas\apps\enterpise\_info($frmData['ent_id'], $frmData['own_id']);
	if($ent == false)  {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的项目不存在。');
		return false;
	}

	// find group name
	$sql = "select `group_id` from cloud_group where `disabled` < " . TB_ROW_DEL_VALUE . " ";
	$sql .="and `ent_id` = ? and `name` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $frmData['ent_id'], \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['name'], \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) > 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '操作失败：“' . $frmData['name'] . '”用户组已存在');
		return false;
	}

	$frmData['ent'] = $ent->name;
	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	$nidb->beginTransaction();
	try {
		if(insert_data('cloud_group', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
		$frmData['group_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}


	if(\raas\apps\enterpise\_group_count_inc($frmData['ent_id'], 1) === false){
		$nidb->rollBack();
		return false;
	}
	if(\raas\apps\entowner\_group_count_inc($frmData['own_id'], 1) === false){
		$nidb->rollBack();
		return false;
	}
	if(\raas\apps\counter\_group_count_inc(COUNTER_RAAS_ID, 1) === false) {
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();
//	synclog_push(RA_SYNC_GROUP, $frmData['group_id']);
	raaseye_group_add($frmData);
	
	return $frmData;
}

