<?php
namespace raas\apps\group;


function select($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	if(format_and_push($data, 'disabled', $optional, '', 'int', false))
		$where_str .= "`disabled` = " . $optional['disabled'] . " and ";
	else
		$where_str .= "`disabled` < " . TB_ROW_DEL_VALUE . " and ";
	
	if(format_and_push($data, 'upstartm', $optional, '', 'int', false))
		$where_str .= "`update_time` >= " . $optional['upstartm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false)) 
		$where_str .= "`update_time` < " . $optional['upendtm']  . " and ";

	if(format_and_push($data, 'startm', $optional, '', 'int', false))
		$where_str .= "`create_time` >= " . $optional['startm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false))
		$where_str .= "`create_time` < " . $optional['endtm']  . " and ";

	// set custom options
	if(format_and_push($data, 'own_id', $optional, '', 'int', false))
		$where_str .= "`own_id` = " . $optional['own_id']  . " and ";
	
	if(format_and_push($data, 'ent_id', $optional, '', 'int', false))
		$where_str .= "`ent_id` = " . $optional['ent_id']  . " and ";

	if(format_and_push($data, 'group_id', $optional, '', 'int', false))
		$where_str .= "`group_id` = " . $optional['group_id']  . " and ";
	
	if(format_and_push($data, 'name', $optional, '', 'string', false)) {
		$where_str .= "`name` = ? and ";
		array_push($values, $optional['name']);
	}
	if(format_and_push($data, 'ent', $optional, '', 'string', false)) {
		$where_str .= "`ent` = ? and ";
		array_push($values, $optional['ent']);
	}

	if(format_and_push($data, 'remark', $optional, '', 'string', false)) {
		$where_str .= "`remark` = ? and ";
		array_push($values, "%" . $optional['remark'] . "%");
	}

	if(isset($data['search'])) {
		if(format_and_push($data['search'], 'value', $optional, 'search', 'string', false)) {
			$where_str .= "(`name` like ? or `remark` like ?) and ";
			array_push($values, "%" . $optional['search'] . "%");
			array_push($values, "%" . $optional['search'] . "%");
		}
	}

	// 限制设置了负责企业的操作员只能访问其负责的企业
	if(gettype($user) == 'object' && $user->entcnt > 0) {
		$sql = "select group_concat(`group_id`) as `ids` from cloud_user_group_list where `user_id` = ? group by `ent_id`";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $user->user_id, \PDO::PARAM_INT);
			if($sth->execute())
				$row = $sth->fetch(\PDO::FETCH_ASSOC);
			else
				$row = false;
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if($row && empty($row['ids']) == false) {
			$where_str .= '`group_id` in (' . $row['ids'] . ') and ';
		}
	}

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);


	$order_map = array(
		"group_id",
		"ent_id",
		"own_id",
		"name",
		"ent",
		"disabled",
		"service",
		"groupcnt",
		"startip",
		"endip",
		"rate_out",
		"rate_in",
		"dns0",
		"dns1",
		"maxonlinetm",
		"clntepa",
		"ifname",
		"vlan_in",
		"vlan_out",
		"acctcnt",
		"update_time",
		"create_time",
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}
	
	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = '';

	$sql = "select count(*) as `total` from cloud_group $where_str";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0) {
		$result = array (
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$keys = '*';
	$sql = "select $keys from cloud_group $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

