<?php
namespace raas\apps\package;


function budget($data)
{
	global $nidb;

	$optional = array();

	if(format_and_push($data, 'package_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_EXP, __function__, '无效的套餐。');
		return false;
	}
	if(format_and_push($data, 'acct_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_EXP, __function__, '无效的用户帐号。');
		return false;
	}
	if(format_and_push($data, 'own_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_EXP, __function__, '归属者不存在。');
		return false;
	}
	if(format_and_push($data, 'count', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_EXP, __function__, '购买数量不能为空。');
		return false;
	}
	if($optional['count'] <= 0) {
		set_errmsg(MSG_LEVEL_EXP, __function__, '购买数量不能小于1。');
		return false;
	}

	// 获取用户帐号
	$sql = "select * from cloud_account where `acct_id` = ? and `own_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['acct_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['own_id'], \PDO::PARAM_INT);
		$sth->execute();

		$uinfo = $sth->fetch(\PDO::FETCH_LAZY);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(!$uinfo) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "指定的用户帐号不存在！");
		return false;
	}

	// 获取套餐
	$sql = "select item.* from cloud_package_group as `grp`";
	$sql.= " inner join cloud_package_item as `item`";
	$sql.= " on item.`package_id` = grp.`package_id`";
	$sql.= " where grp.`group_id` = ? and item.`own_id` = ? and item.`package_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $uinfo->group_id, \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(3, $optional['package_id'], \PDO::PARAM_INT);
		$sth->execute();
		$pkg = $sth->fetch(\PDO::FETCH_LAZY);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$pkg) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "指定的套餐不存在！");
		return false;
	}

	// 检查用户时间，如果到期则使用当前时间：
	// 1、如果指定了起始时间，则使用指定起始时间。
	// 2、如果还未到期，则无法指定起始时间。
	$ret = array(
		'action'		=> 'addtm',
		'acct_id'		=> $uinfo->acct_id,
		'group_id'		=> $uinfo->group_id,
		'package_id'	=> $pkg->package_id,
		'start'			=> 0,
		'expire'		=> 0,
		'tmtype'		=> 0,
		'tmlong'		=> 0,
		'count'			=> $optional['count'],
		'expense'		=> $pkg->expense,
		'src_time'		=> array (
			'start'		=> $uinfo->start_time,
			'expire'	=> $uinfo->expire_time
		),
	);

	// $today = new DateTime(date('Y-m-d'));
	// $today->getTimestamp();
	$currtm = time();
	if($uinfo->expire_time > $currtm) {
		$ret['start']		= $uinfo->start_time;
		$ret['expire']		= $uinfo->expire_time;
		$ret['tmtype']		= $pkg->tmtype;
		$ret['tmlong']		= $pkg->tmlong;
	}
	else {
		$ret['start']		= $currtm;
		$ret['expire']		= $currtm;
		$ret['tmtype']		= $pkg->tmtype;
		$ret['tmlong']		= $pkg->tmlong;
	}
	$ret['tmlong']		= $pkg->tmlong;

	// 计算时间
	$expire = new \DateTime(date('Y-m-d H:i:s', $ret['expire']));
	switch($ret['tmtype']) {
		case 1: // 分
			$expire->add(new \DateInterval('P' . ($ret['tmlong'] * 60 * $ret['count']) . 'S'));
			break;
		case 2: // 时
			$expire->add(new \DateInterval('P' . ($ret['tmlong'] * $ret['count']) . 'H'));
			break;
		case 3: // 天
			$expire->add(new \DateInterval('P' . ($ret['tmlong'] * $ret['count']) . 'D'));
			break;
		case 4: // 月
			$expire->add(new \DateInterval('P' . ($ret['tmlong'] * $ret['count']) . 'M'));
			break;
		case 5: // 年
			$expire->add(new \DateInterval('P' . ($ret['tmlong'] * $ret['count']) . 'Y'));
			break;
		default:
			break;
	}
	$ret['expire']	= $expire->getTimestamp();

	return $ret;
}

