<?php
namespace raas\apps\package;

function _create($frmData)
{
	global $nidb;


	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	try {
		if(insert_data('cloud_package_item', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$frmData['package_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $frmData;
}

function create($data)
{
	global $nidb;

	$filter = array(
		'name'		=> new \FilterVarStr(true),
		'own_id'	=> new \FilterVarID(true),
		'tmlong'	=> new \FilterVarInt(),
		'tmtype'	=> new \FilterVarInt(),
		'expense'	=> new \FilterVarInt(),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(empty($frmData['name'])) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '名称不能为空！');
		return false;
	}

	// find name is exist
	$sql = "select `package_id` from cloud_package_item where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql .=" and `own_id` = ? and `name` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['name'], \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) > 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '“' . $frmData['name'] . '”已存在');
		return false;
	}

	$nidb->beginTransaction();
	
	if(($frmData = _create($frmData)) === false) {
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();

	return $frmData;
}

