<?php
namespace raas\apps\package\group;

function add($data)
{
	global $nidb, $user;

	$filter = array(
		'package_id'	=> new \FilterVarID(true),
		'group_id'		=> new \FilterVarID(true),
		'own_id'		=> new \FilterVarID(true),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	// 检查用户是否是自己组织的 
	$sql = "select `group_id`, `own_id` from cloud_group where `own_id` = ?, `group_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $frmData['group_id'], \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_LAZY);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的用户组！');
		return false;
	}


	unset($frmData['own_id']);

	if(insert_data('cloud_package_group', $frmData) === false) {
		$errmsg = implode(' ', $nidb->errorInfo());
		set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
		$nidb->rollBack();
		return false;
	}

	return true;
}

function del($data)
{
	global $nidb, $user;

	$filter = array(
		'package_id'	=> new \FilterVarID(true),
		'group_id'		=> new \FilterVarID(true),
		'own_id'		=> new \FilterVarID(true),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	// 检查用户是否是自己组织的 
	$sql = "select `group_id`, `own_id` from cloud_group where `own_id` = ?, `group_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $frmData['group_id'], \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_LAZY);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的用户组！');
		return false;
	}

	$sql = "delete from cloud_role_user where `group_id` = ? and `package_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['user_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $frmData['role_id'], \PDO::PARAM_INT);

		$sth->execute();
		$count = $sth->rowCount();
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function lst($data)
{
	global $nidb;

	$filter = array(
		'group_id'		=> new \FilterVarID(true),
		'own_id'		=> new \FilterVarID(true),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}


	// 检查角色是否是自己组织的
	$sql = "select `package_id` from cloud_package_group where `group_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['group_id'], \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_OBJ);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($rows === false || count($rows) <= 0) 
		return (object)array();

	return $rows;
}

function get_packages($data)
{
	// 获取指定套餐列表
	global $nidb;

	if(isset($data['ids']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，参数无效！');
		return false;
	}
	if(is_array($data['ids']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，指定的套餐不正确！');
		return false;
	}

	if(isset($data['own_id']) == false
	|| ($own_id = intval($data['own_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的归属，请先登陆。');
		return false;
	}

	// 获取用户 token 
	$sql = "select group.`group_id`, item.`package_id`, item.`name`, item.`disabled` from cloud_package_group as `group`";
	$sql.= " inner join cloud_package_item as `item`";
	$sql.= " on group.`package_id` = item.`package_id`";
	$sql.= " where group.`package_id` in (" . substr(str_repeat(',?', count($data['ids'])), 1) . ")";
	$sql.= " and item.`own_id` = ?";

	array_push($data['ids'], $own_id);
	
	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($data['ids']);
		
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($rows === false || count($rows) <= 0) 
		return (object)array();

	return $rows;
}

function lst_ent($data)
{
	// 获取所有企业与用户组列表
	global $nidb;

	if(isset($data['own_id']) == false
	|| ($own_id = intval($data['own_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的归属，请先登陆。');
		return false;
	}

	// 获取用户 token 
	$sql = "select gp.`group_id`, ent.`ent_id`, ent.`name` as `ent`, gp.`name` from cloud_group as `gp`";
	$sql.= " inner join cloud_enterpise as `ent`";
	$sql.= " on gp.`ent_id` = ent.`ent_id`";
	$sql.= " where ent.`disabled` < " . TB_ROW_DEL_VALUE;
	$sql.= " and gp.`disabled` < " . TB_ROW_DEL_VALUE;
	$sql.= " and gp.`own_id` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $own_id, \PDO::PARAM_INT);
		$sth->execute();

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($rows === false || count($rows) <= 0) 
		return (object)array();

	return $rows;
}

function get_option($data)
{
	global $nidb;
	
	if(is_array($data['ids']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，指定的用户组不正确！');
		return false;
	}
	
	if(isset($data['own_id']) == false
	|| ($own_id = intval($data['own_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的归属，请先登陆。');
		return false;
	}

	if(isset($data['package_id']) == false
	|| ($package_id = intval($data['package_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的套餐。');
		return false;
	}
	
	// 获取用户 token 
	$sql = "select gp.`group_id`, gp.`package_id`, item.`disabled` from cloud_package_item as `item`";
	$sql.= " inner join cloud_package_group as `gp`";
	$sql.= " on gp.`package_id` = item.`package_id`";
	$sql.= " where gp.`group_id` in (" . substr(str_repeat(',?', count($data['ids'])), 1) . ")";
	$sql.= " and item.`own_id` = ? and item.`package_id` = ?";

	array_push($data['ids'], $own_id);
	array_push($data['ids'], $package_id);
	
	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($data['ids']);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($rows === false || count($rows) <= 0) 
		return (object)array();

	return $rows;
}

function set_option($data)
{
	global $nidb;
	
	if(is_array($data['ids']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，指定的用户组不正确！');
		return false;
	}
	if(isset($data['own_id']) == false
	|| ($own_id = intval($data['own_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的归属，请先登陆。');
		return false;
	}
	if(isset($data['package_id']) == false
	|| ($package_id = intval($data['package_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的套餐。');
		return false;
	}
	if(isset($data['checked']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的套餐。');
		return false;
	}
	$checked = intval($data['checked']);
	
	// 检查套餐是否存在 
	$sql = "select `package_id` from cloud_package_item";
	$sql.= " where `package_id` = ? and `own_id` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $package_id, \PDO::PARAM_INT);
		$sth->bindParam(2, $own_id, \PDO::PARAM_INT);
		$sth->execute();

		$row = $sth->fetch(\PDO::FETCH_LAZY);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的套餐！');
		return false;
	}

	// 设置套餐
	$count = 0;
	$nidb->beginTransaction();
	try {
		if($checked) {
			// 选中
			$sql = "insert into `cloud_package_group` (`package_id`,`group_id`) values (?,?) on duplicate key update `package_id` = ?, `group_id` = ?";
			foreach($data['ids'] as $id) {
				$sth = $nidb->prepare($sql);
				$sth->bindParam(1, $package_id, \PDO::PARAM_INT);
				$sth->bindParam(2, $id, \PDO::PARAM_INT);
				$sth->bindParam(3, $package_id, \PDO::PARAM_INT);
				$sth->bindParam(4, $id, \PDO::PARAM_INT);
				$sth->execute();
				$count += $sth->rowCount();
			}
		}
		else {
			// 未选
			$sql = "delete from cloud_package_group where `group_id` in (" . substr(str_repeat(',?', count($data['ids'])), 1) . ") and `package_id` = ?";
			array_push($data['ids'], $package_id);

			$sth = $nidb->prepare($sql);
			$sth->execute($data['ids']);
			$count -= $sth->rowCount();
		}
		
		if($count != 0) {
			// 统计引用数
/*
			$sql = "select count(*) as `count` from cloud_package_group where `package_id` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $package_id, \PDO::PARAM_INT);
			$sth->execute();
			$row = $sth->fetch(\PDO::FETCH_LAZY);
			if($row)
				$count = $row->count;
			else
				$count = 0;
*/
			// 更新计数器
			$sql = "update cloud_package_item set `count` = ?  where `package_id` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $count, \PDO::PARAM_INT);
			$sth->bindParam(2, $package_id, \PDO::PARAM_INT);
			$sth->execute();
		}
		$nidb->commit();
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	return true;
}
