<?php
namespace raas\apps\presale_card\item;

function _create($frmData)
{
	global $nidb;


	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	try {
		if(insert_data('cloud_presale_card', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$frmData['psc_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $frmData;
}

function comp_func_card($a, $b)
{
    return $a <=> $b->psi_code;
//	if ($a->psi_code === $b->psi_code) return 0;
//	return ($a->psi_code > $b->psi_code) ? 1:-1;
}


function card_batbuild($data)
{
	global $nidb;

	$filter = array(
		'psc_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
		'ent_id'	=> new \FilterVarInt(),
		'mkcnt'		=> new \FilterVarInt(true),
		'start_time'=> new \FilterVarStr(true),
		'card_tmlong'=> new \FilterVarInt(true),
		'card_tmtype'=> new \FilterVarInt(true),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if($data['mkcnt'] == '') {
		set_errmsg(MSG_LEVEL_DEF, __function__, '生成数量不能为空！');
		return false;
	}
	if($frmData['mkcnt'] <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '一次生成不能小于等于0条！');
		return false;
	}
	if($frmData['mkcnt'] > 1000) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '一次生成不能大于1000条！');
		return false;
	}

	if($frmData['card_tmtype'] > 0) {
		if($frmData['card_tmlong'] <= 0) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '卡有效时长不能为空或小于等于0！');
			return false;
		}
		if(empty($frmData['start_time'])) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '开始使用日不能为空！');
			return false;
		}
		
		// 计算时间
		$expire = new \DateTime($frmData['start_time']);
		if($expire === FALSE) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '起始时间不正确！');
			return false;
		}
		$frmData['start_time'] = $expire->getTimestamp();
		$card_tmlong = abs($frmData['card_tmlong']);
		switch($frmData['card_tmtype']) {
			case 1: // 分
				$expire->add(new \DateInterval('PT' . ($card_tmlong * 60) . 'S'));
				break;
			case 2: // 时
				$expire->add(new \DateInterval('PT' . ($card_tmlong * 3600) . 'S'));
				break;
			case 3: // 天
				$expire->add(new \DateInterval('P' . $card_tmlong . 'D'));
				break;
			case 4: // 月
				$expire->add(new \DateInterval('P' . $card_tmlong . 'M'));
				break;
			case 5: // 年
				$expire->add(new \DateInterval('P' . $card_tmlong . 'Y'));
				break;
			default:
				break;
		}

		$frmData['expire_time'] = $expire->getTimestamp();
	}
	else {
		$frmData['start_time'] = 0;
		$frmData['expire_time'] = 0;
	}

	// find name is exist
	$keys = "`psc_id`,
`ent_id`,
`group_id`,
`own_id`,
`name`,
`expense`,
`tmtype`,
`tmlong`,
`rate_out`,
`rate_in`,
`update_time`,
`create_time`";

	$sql = "select {$keys} from cloud_presale_card where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql .=" and `own_id` = ? and `psc_id` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['psc_id'], \PDO::PARAM_INT);

		$sth->execute();
		$psc_card = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(!$psc_card || empty($psc_card)) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的充值卡不存在。');
		return false;
	}

	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	$psc_card['own_id'] = $frmData['own_id'];
//	if(isset($frmData['ent_id']))
//		$psc_card['ent_id'] = $frmData['ent_id'];

	$code_data = array();
	$count = $frmData['mkcnt'];
	$loopmax = $count;

	try {
		$map = 'abcdefghijklmnopqrstuvwxyz1234567890';
		$l = strlen($map) - 1;

		while(count($code_data) < $count && $loopmax > 0) {
			$rand_data = array();
			$cnt = min($count - count($code_data), 100);
			do {
				$nc = 8;
				$r = '';
				while($nc-- > 0) $r .= $map{mt_rand(0, $l)};
				if(isset($code_data[$r]))
					continue;
				$rand_data[$r] = 1;
			} while(count($rand_data) < $cnt);

			// search
			$codemask = str_repeat(",?", count($rand_data));
			if(count($rand_data) > 0)
				$codemask = substr($codemask, 1);
			$sql = "select `psi_code` from cloud_presale_item where `own_id` = {$psc_card['own_id']} and `psi_code` in ({$codemask})";

			$sth = $nidb->prepare($sql);
			$sth->execute(array_keys($rand_data));

			$oldcode = $sth->fetchAll(\PDO::FETCH_CLASS);
			if($oldcode && count($oldcode)){
				$result = array_udiff_assoc(array_keys($rand_data), $oldcode, comp_func_card);
				$code_data = array_merge($code_data, $result);
			}
			else
				$code_data = array_merge($code_data, $rand_data);

			$loopmax--;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$nidb->beginTransaction();
	try {
		$psc_card['create_time'] = time();
		$psc_card['update_time'] = $psc_card['create_time'];
		$psc_card['start_time']	 = $frmData['start_time'];
		$psc_card['expire_time'] = $frmData['expire_time'];

		foreach($code_data as $key => $val) {
			$psc_card['psi_code'] = $key;

			if(insert_data('cloud_presale_item', $psc_card) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				$nidb->rollBack();
				return false;
			}
		}

		$sql = 'update cloud_presale_card set `count` = `count` + ' . count($code_data);
		$sql.= ' where `psc_id` = ?';

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['psc_id'], \PDO::PARAM_INT);
		$sth->execute();

		$nidb->commit();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	return $count;
}

function card_batbuild_array($data)
{
	global $nidb;

	$filter = array(
		'psc_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
		'ent_id'	=> new \FilterVarInt(),
		'mkcnt'		=> new \FilterVarInt(true),
		'start_time'=> new \FilterVarStr(true),
		'card_tmlong'=> new \FilterVarInt(true),
		'card_tmtype'=> new \FilterVarInt(true),
		'ordernum'	=> new \FilterVarStr(),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if($data['mkcnt'] == '') {
		set_errmsg(MSG_LEVEL_DEF, __function__, '生成数量不能为空！');
		return false;
	}
	if($frmData['mkcnt'] <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '一次生成不能小于等于0条！');
		return false;
	}
	if($frmData['mkcnt'] > 1000) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '一次生成不能大于1000条！');
		return false;
	}

	if($frmData['card_tmtype'] > 0) {
		if($frmData['card_tmlong'] <= 0) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '卡有效时长不能为空或小于等于0！');
			return false;
		}
		if(empty($frmData['start_time'])) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '开始使用日不能为空！');
			return false;
		}
		
		// 计算时间
		$expire = new \DateTime($frmData['start_time']);
		if($expire === FALSE) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '起始时间不正确！');
			return false;
		}
		$frmData['start_time'] = $expire->getTimestamp();
		$card_tmlong = abs($frmData['card_tmlong']);
		switch($frmData['card_tmtype']) {
			case 1: // 分
				$expire->add(new \DateInterval('PT' . ($card_tmlong * 60) . 'S'));
				break;
			case 2: // 时
				$expire->add(new \DateInterval('PT' . ($card_tmlong * 3600) . 'S'));
				break;
			case 3: // 天
				$expire->add(new \DateInterval('P' . $card_tmlong . 'D'));
				break;
			case 4: // 月
				$expire->add(new \DateInterval('P' . $card_tmlong . 'M'));
				break;
			case 5: // 年
				$expire->add(new \DateInterval('P' . $card_tmlong . 'Y'));
				break;
			default:
				break;
		}

		$frmData['expire_time'] = $expire->getTimestamp();
	}
	else {
		$frmData['start_time'] = 0;
		$frmData['expire_time'] = 0;
	}

	// find name is exist
	$keys = "`psc_id`,
`ent_id`,
`group_id`,
`own_id`,
`name`,
`expense`,
`tmtype`,
`tmlong`,
`rate_out`,
`rate_in`,
`update_time`,
`create_time`";

	$sql = "select {$keys} from cloud_presale_card where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql .=" and `own_id` = ? and `psc_id` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['psc_id'], \PDO::PARAM_INT);

		$sth->execute();
		$psc_card = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(!$psc_card || empty($psc_card)) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的充值卡不存在。');
		return false;
	}

	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	$psc_card['own_id'] = $frmData['own_id'];
	if(isset($frmData['ent_id']))
		$psc_card['ent_id'] = $frmData['ent_id'];

	$code_data = array();
	$count = $frmData['mkcnt'];
	$loopmax = $count;

	try {
		$map = 'abcdefghijklmnopqrstuvwxyz1234567890';
		$l = strlen($map) - 1;

		while(count($code_data) < $count && $loopmax > 0) {
			$rand_data = array();
			$cnt = min($count - count($code_data), 100);
			do {
				$nc = 8;
				$r = '';
				while($nc-- > 0) $r .= $map{mt_rand(0, $l)};
				if(isset($code_data[$r]))
					continue;
				$rand_data[$r] = 1;
			} while(count($rand_data) < $cnt);

			// search
			$codemask = str_repeat(",?", count($rand_data));
			if(count($rand_data) > 0)
				$codemask = substr($codemask, 1);
			$sql = "select `psi_code` from cloud_presale_item where `own_id` = {$psc_card['own_id']} and `psi_code` in ({$codemask})";

			$sth = $nidb->prepare($sql);
			$sth->execute(array_keys($rand_data));

			$oldcode = $sth->fetchAll(\PDO::FETCH_CLASS);
			if($oldcode && count($oldcode)){
				$result = array_udiff_assoc(array_keys($rand_data), $oldcode, comp_func_card);
				$code_data = array_merge($code_data, $result);
			}
			else
				$code_data = array_merge($code_data, $rand_data);

			$loopmax--;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$cardinfo = array();
	$nidb->beginTransaction();
	try {
		$psc_card['create_time'] = time();
		$psc_card['update_time'] = $psc_card['create_time'];
		$psc_card['start_time']	 = $frmData['start_time'];
		$psc_card['expire_time'] = $frmData['expire_time'];
		$psc_card['ordernum']	 = $frmData['ordernum'];

		foreach($code_data as $key => $val) {
			$psc_card['psi_code'] = $key;

			if(insert_data('cloud_presale_item', $psc_card) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				$nidb->rollBack();
				return false;
			}
			array_push($cardinfo, $key);
		}

		$sql = 'update cloud_presale_card set `count` = `count` + ' . count($code_data);
		$sql.= ' where `psc_id` = ?';

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['psc_id'], \PDO::PARAM_INT);
		$sth->execute();

		$nidb->commit();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	return $cardinfo;
}
