<?php
namespace raas\apps\presale_card\item;

ni_app_load('package', 'budget');
ni_app_load('account_book', 'create');
ni_app_load('account', 'create');
ni_app_load('account', 'save');
ni_app_load('group', 'info');
ni_app_load('enterpise', 'info');
ni_app_load('account', 'info');
ni_app_load('presale_card', 'card-info');


function creditget($data, $getCard = false)
{
	global $nidb;

	$optional = array();

	if(format_and_push($data, 'code', $optional, 'psi_code', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的卡号。');
		return false;
	}
	if(format_and_push($data, 'acct_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的用户帐号。');
		return false;
	}
	if(format_and_push($data, 'own_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '归属者不存在。');
		return false;
	}

	// 获取用户帐号
	$sql = "select * from cloud_account where `acct_id` = ? and `own_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['acct_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['own_id'], \PDO::PARAM_INT);
		$sth->execute();

		$uinfo = $sth->fetch(\PDO::FETCH_LAZY);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(!$uinfo) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "指定的用户帐号不存在！");
		return false;
	}

	// 获取卡号
	$sql = "select * from cloud_presale_item where `own_id` = ? and `psi_code` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $uinfo->own_id, \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['psi_code'], \PDO::PARAM_STR);
		$sth->execute();
		$card = $sth->fetch(\PDO::FETCH_LAZY);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$card) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "指定的卡号不存在！");
		return false;
	}
	
	if($card->ent_id > 0 && $card->ent_id != $uinfo->ent_id) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "无此卡号。");
		return false;
	}

	$now = time();
	if($card->expire_time > 0) {
		if($now < $card->start_time) {
			set_errmsg(MSG_LEVEL_DEF, __function__, "请在“" . date('Y-m-d H:i:s', $card->start_time) . "”至“" . date('Y-m-d H:i:s', $card->expire_time) . "”期间使用。");
			return false;
		}
		if($now > $card->expire_time) {
			set_errmsg(MSG_LEVEL_DEF, __function__, "此卡已在“" . date('Y-m-d H:i:s', $card->expire_time) . "”到期。");
			return false;
		}
	}

	// 检查用户时间，如果到期则使用当前时间：
	// 1、如果指定了起始时间，则使用指定起始时间。
	// 2、如果还未到期，则无法指定起始时间。
	$ret = array(
		'action'		=> 'addtm',
		'acct_id'		=> $uinfo->acct_id,
		'account'		=> $uinfo->account,
		'group_id'		=> $uinfo->group_id,
		'psi_code'		=> $card->psi_code,
		'rate_out'		=> $card->rate_out,
		'rate_in'		=> $card->rate_in,
		'start'			=> 0,
		'expire'		=> 0,
		'tmtype'		=> 0,
		'tmlong'		=> 0,
		'expense'		=> $card->expense,
		'src_time'		=> array (
			'start'		=> $uinfo->start_time,
			'expire'	=> $uinfo->expire_time
		),
	);
	
	// 返回获取到的卡号
	if($getCard) {
		$ret['card']	= (array)$card;
		unset($ret['card']['queryString']);
	}

	// $today = new DateTime(date('Y-m-d'));
	// $today->getTimestamp();
	$currtm = time();
	if($uinfo->expire_time > $currtm) {
		$ret['start']		= $uinfo->start_time;
		$ret['expire']		= $uinfo->expire_time;

		// 检查用户上下行限速是否一致。
		if($uinfo->rate_in > 0 || $uinfo->rate_out > 0) {
			if($ret['rate_in'] != $uinfo->rate_in
			|| $ret['rate_out'] != $uinfo->rate_out) {
				// set_errmsg(MSG_LEVEL_DEF, __function__, "带宽不匹配，无法充值。");
				set_errmsg(MSG_LEVEL_DEF, __function__, "带宽不匹配，无法充值。");
				return false;
			}
		}
	}
	else {
		$ret['start']		= $currtm;
		$ret['expire']		= $currtm;
	}

	$ret['tmtype']		= $card->tmtype;
	$ret['tmlong']		= $card->tmlong;

	// 计算时间
	$expire = new \DateTime(date('Y-m-d H:i:s', $ret['expire']));
	switch($ret['tmtype']) {
		case 1: // 分
			$expire->add(new \DateInterval('PT' . ($ret['tmlong'] * 60) . 'S'));
			break;
		case 2: // 时
			$expire->add(new \DateInterval('PT' . ($ret['tmlong'] * 3600) . 'S'));
			break;
		case 3: // 天
			$expire->add(new \DateInterval('P' . $ret['tmlong'] . 'D'));
			break;
		case 4: // 月
			$expire->add(new \DateInterval('P' . $ret['tmlong'] . 'M'));
			break;
		case 5: // 年
			$expire->add(new \DateInterval('P' . $ret['tmlong'] . 'Y'));
			break;
		default:
			break;
	}
	$ret['expire']	= $expire->getTimestamp();

	return $ret;
}

function fmtbps($bps)
{
	if ($bps >= 1000000) // (1000 * 1000))
		return sprintf("%uM", $bps / 1000000);
	else
	if ($bps >= 1000)
		return sprintf("%uK", $bps / 1000);

	return sprintf("%u", $bps);	
}

function credit($data, $beginTransaction = true)
{
	global $nidb;

	$logger = array();
	$optional = creditget($data, true);
	if($optional === false) return false;

	format_and_push($data, 'user_id', $optional, '', 'int', true);

	$grp = \raas\apps\group\info(array(
		'own_id'	=> $data['own_id'],
		'group_id'	=> $optional['group_id']));
	if($grp === false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "无效的用户组。");
		return false;
	}

	if($beginTransaction)
		$nidb->beginTransaction();

	try {

		$sql = "delete from cloud_presale_item where `psc_id` = ? and `own_id` = ? and `psi_code` = ?";

		$sth = $nidb->prepare($sql);

		$sth->bindParam(1, $optional['card']['psc_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['card']['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(3, $optional['card']['psi_code'], \PDO::PARAM_STR);
		$sth->execute();


		$where = array('acct_id' => $optional['acct_id']);
		$frmData = array(
			'start_time'	=> $optional['start'],
			'expire_time'	=> $optional['expire'],
			'rate_out'		=> $optional['rate_out'],
			'rate_in'		=> $optional['rate_in'],
		);

		// save account
		if(update_data('cloud_account', $frmData, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
		
		$sql = 'update cloud_presale_card set `usedcnt` = `usedcnt` + 1';
		$sql.= ' where `psc_id` = ?';

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['card']['psc_id'], \PDO::PARAM_INT);
		$sth->execute();

		$nidb->commit();
		
		$ent = \raas\apps\enterpise\_info($grp->ent_id);
		// 正常使用
		$optional['card']['operation'] = 2;
		$optional['card']['update_time'] = time();
		$optional['card']['account'] = $optional['account'] . '@' .  $ent->domain;

		if(insert_data('cloud_presale_logger', $optional['card']) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
	}
	catch (PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	if(isset($data['ordernum']) && empty($data['ordernum']) == false)
		$ordernum = " 商户订单号：{$data['ordernum']}";
	else
		$ordernum = '';

	// save logger
	$logger['own_id']	= $grp->own_id;
	$logger['ent_id']	= $grp->ent_id;
	$logger['user_id']	= $optional['user_id'];
	$logger['acct_id']	= $optional['acct_id'];
	$logger['expense']	= $optional['expense'];
	$logger['action']	= $optional['action'];
	$logger['remark']	= '使用充值卡：' . $optional['card']['psi_code'] . ' 限速bps：' . fmtbps($optional['rate_in']) . ' / ' . fmtbps($optional['rate_out']) . ' [in/out]' . $ordernum;
	$logger['data']		= array(
		'showmode'		=> 0,
		'count'			=> $optional['count'],
		'tmlong'		=> $optional['tmlong'],
		'tmtype'		=> $optional['tmtype'],
		'start'			=> $optional['start'],
		'expire'		=> $optional['expire'],
		'src_time'		=> $optional['src_time'],
		'rate_out'		=> $optional['rate_out'],
		'rate_in'		=> $optional['rate_in'],
	);
	$ret = \raas\apps\account_book\create($logger);

	$info = \raas\apps\account\info($where, 1);
	raaseye_acct_add($info);

	return $ret;
}

function create_acct_and_credit($data)
{
	global $nidb;

	$filter = array(
		'own_id'	=> new \FilterVarID(true),
		'group_id'	=> new \FilterVarID(true),
		'parent_id'	=> new \FilterVarInt(),
		'account'	=> new \FilterVarStr(true),
		'pass'		=> new \FilterVarStr(true),
		'disabled'	=> new \FilterVarInt(),
//		'webuser_id'=> new \FilterVarInt(),
		'pxyacct'	=> new \FilterVarStr(),
		'pxypass'	=> new \FilterVarStr(),
//		'pxyvlan_in'=> new \FilterVarInt(),
//		'pxyvlan_out'=> new \FilterVarInt(),
		'create_time'=> new \FilterVarInt(),
//		'start_time'=> new \FilterVarInt(),
//		'expire_time'=> new \FilterVarInt(),
//		'bind_ipv4'	=> new \FilterVarInt(),
//		'bind_v4mac'=> new \FilterVarStr(),
//		'bind_vlan'	=> new \FilterVarInt(),
//		'rate_out'	=> new \FilterVarKBit(),
//		'rate_in'	=> new \FilterVarKBit(),
//		'maxcnt'	=> new \FilterVarInt(),
		'name'		=> new \FilterVarStr(),
		'idcard'	=> new \FilterVarStr(),
		'phone'		=> new \FilterVarStr(),
		'remark'	=> new \FilterVarStr(),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(empty($frmData['account'])) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不能为空！');
		return false;
	}

	if(isset($data['asacctpassw']) && isset($frmData['pxyacct']) 
	&& empty($frmData['pxyacct']) == false
	&& intval($data['asacctpassw'])) {
		if(isset($frmData['pass'])) 
			$frmData['pxypass'] = $frmData['pass'];
	}

	/*
	if(strpos($frmData['account'], '@') !== false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不能有“@”字符！');
		return false;
	}
	*/
	if(preg_match("/^([a-zA-Z0-9-_.]{2,})$/", $frmData['account'], $match) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号格式不正确！格式应为：a-z，A-Z，0-9和“.”，“_”，至少2位字符组成。');
		return false;
	}

	// find group
	$info = array (
		'group_id'	=> $frmData['group_id'],
		'own_id'	=> $frmData['own_id'],
	);
	if(($grp = \raas\apps\group\info($info)) === false){
		set_errmsg(MSG_LEVEL_DEF, __function__, '请选择一个有效的用户组。');
		return false;
	}

	if(isset($data['check_pxy_acct']) && intval($data['check_pxy_acct'])
	&& isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
		$sql = "select `account` from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
		$sql.=" and `own_id` = ? and `ent_id` = ?";
		$sql.=" and `pxyacct` = ? limit 1";

		try {
			$sth = $nidb->prepare($sql);

			$index = 1;

			$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
			$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if($rows && count($rows) > 0) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '代拨帐号已存在。');
			return 'PXY_ACCT_EXIST';
		}
	}

	// find account is exist
	$sql = "select group_id from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql.= " and `ent_id` = ? and `account` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
		$sth->bindParam($index++, $frmData['account'], \PDO::PARAM_STR);

		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_LAZY);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if($row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '“' . $frmData['account'] . '” EXIST');
		return false;
	}

	$frmData['own_id'] = $grp->own_id;
	$frmData['ent_id'] = $grp->ent_id;
	
	$frmData['group'] = $grp->name;
	$frmData['maxcnt'] = 3;

	$nidb->beginTransaction();

	try {
		if(($frmData = \raas\apps\account\_create($frmData)) === false) {
			$nidb->rollBack();
			return false;
		}
		
		if(isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
			try {
				$where = array('pxyacct' => $frmData['pxyacct']);
				if(update_data('cloud_account', array('pxypass' => $frmData['pxypass']), $where) === false) {
					$errmsg = implode(' ', $nidb->errorInfo());
					set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
					$nidb->rollBack();
					return false;
				}
				
				$sql = "select * from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
				$sql.=" and `own_id` = ? and `ent_id` = ?";
				$sql.=" and `pxyacct` = ? and `acct_id` != ?";

				$sth = $nidb->prepare($sql);

				$index = 1;

				$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
				$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
				$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);
				$sth->bindParam($index++, $frmData['acct_id'], \PDO::PARAM_INT);

				$sth->execute();
				$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
			}
			catch (\NiDBException $e) {
				$nidb->rollBack();
				set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
				return false;
			}
		}
		else {
			try {
				$sql = "select * from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
				$sql.=" and `own_id` = ? and `ent_id` = ?";
				$sql.=" and `acct_id` != ?";

				$sth = $nidb->prepare($sql);

				$index = 1;

				$sth->bindParam($index++, $frmData['own_id'], \PDO::PARAM_INT);
				$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
				$sth->bindParam($index++, $frmData['acct_id'], \PDO::PARAM_INT);

				$sth->execute();
				$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
			}
			catch (\NiDBException $e) {
				$nidb->rollBack();
				set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
				return false;
			}
		}

		$data['acct_id'] = $frmData['acct_id'];
		$ret = credit($data, false);
		if($rows && $ret) {
			foreach($rows as $row) {
				 raaseye_acct_add($row);
			}
		}

		return $ret;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
}
