<?php
namespace raas\apps\service;

// _group_count_inc load
ni_app_load('group', 'find');
ni_app_load('account', 'create');
ni_app_load('account', 'disable');
ni_app_load('account', 'enable');
ni_app_load('account', 'save');
ni_app_load('enterpise', 'find');

function account_sync($data)
{
	global $nidb;
	
	// enterpise is exist?
	if(format_and_push($data, 'account', $data, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'err: account can\'t empty.');
		return false;
	}
	if(preg_match("/^([a-zA-Z0-9-_.]{2,})@([a-zA-Z0-9-_.]{2,})$/", $data['account'], $match) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号格式不正确！格式应为：a-z，A-Z，0-9和“.”，“_”，“@”组成');
		return false;
	}
	
	if(($ent = \raas\apps\enterpise\_finddomain($match[2])) === false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的项目不存在，添加失败！');
		return false;
	}
	if($ent->disabled) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的项目已禁用，操作无法继续完成！');
		return false;
	}
	$data['account'] = $match[1];
	$data['own_id'] = $ent->own_id;

	// group is exist?
	if(format_and_push($data, 'group', $data, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'err: group can\'t empty.');
		return false;
	}
	if(($group = \raas\apps\group\find(array('ent_id' => $ent->ent_id, 'name' => $data['group']))) === false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的用户组不存在，添加失败！');
		return false;
	}
	if($group->disabled) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的用户组已禁用，操作无法继续完成！');
		return false;
	}
	if($group->service == 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '不支持的操作！');
		return false;
	}
	
	// 会话数不限
	$data['maxcnt'] = 0;

	$data['group_id'] = $group->group_id;
	if(isset($data['iwanacct']) && empty(($acct = trim($data['iwanacct']))) == false) {
		$sql = "select * from cloud_account where `account` <> ? and `iwanacct` = ? and `ent_id` = ?";
		try {
			if($own_id > 0) $sql .= ' and `own_id` = ?';
			$sth = $nidb->prepare($sql);

			$index = 1;
			$sth->bindParam($index++, $data['account'], \PDO::PARAM_STR);
			$sth->bindParam($index++, $acct, \PDO::PARAM_STR);
			$sth->bindParam($index++, $ent->ent_id, \PDO::PARAM_INT);
			if($own_id > 0) 
				$sth->bindParam($index++, $own_id, \PDO::PARAM_INT);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		$account = array(
			'acct_id' => 0,
			'ent_id' => 0,
			'own_id' => $ent->own_id
		);

		foreach($rows as $row) {
			$account['acct_id'] = $row->acct_id;
			$account['ent_id'] = $row->ent_id;
			if(\raas\apps\account\disable($account) === false) {
				set_errmsg(MSG_LEVEL_DEF, __function__, '删除旧的关联帐号失败！');
				return false;
			}
		}
	}

	$sql = "select * from cloud_account where `account` = ? and `ent_id` = ?";
	try {
		if($own_id > 0) $sql .= ' and `own_id` = ?';
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $data['account'], \PDO::PARAM_STR);
		$sth->bindParam($index++, $ent->ent_id, \PDO::PARAM_INT);
		if($own_id > 0) 
			$sth->bindParam($index++, $own_id, \PDO::PARAM_INT);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0) {
		$acct = $rows[0];
		$data['acct_id'] = $acct->acct_id;
		$data['ent_id'] = $acct->ent_id;
		$data['disabled'] = 0;

		// save account
		if(($ret = \raas\apps\account\save($data)) !== false)
			$ret = array('_action_' => 'save');
	}
	else {
		// create account
		if(($ret = \raas\apps\account\create($data)) !== false)
			$ret['_action_'] = 'create';
	}

	return $ret;
}

