<?php
namespace raas\apps\upgrade;

// function
function _getpagid($pagid, $ent_id)
{
	global $nidb, $user;

	// set custom options
	$where_str = "where `disabled` < " . TB_ROW_DEL_VALUE . " and `pagid` = ?";
	$where_str .= " and `ent_id` = ? and `own_id` = ?";

	$sql = "select * from cloud_group $where_str";

	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$own_id = $user->own_id;
		$sth->bindParam($index++, $pagid, \PDO::PARAM_INT);
		$sth->bindParam($index++, $ent_id, \PDO::PARAM_INT);
		$sth->bindParam($index++, $own_id, \PDO::PARAM_INT);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "无效的分组。");
		return false;
	}
	
	return $rows[0];
}

function _import_account_from_panabit($acctfile, $ent_id)
{
	global $user;
	
	$result = array(
		'groupcnt'	=> 0,
		'acctcnt'	=> 0,
		'error'		=> array(),
	);

	$fd = fopen($acctfile["tmp_name"], "r");
	if ($fd) {
		
		ni_app_load('group', 'create');
		ni_app_load('account', 'create');
		while (($buff = fgets($fd, 4096)) !== false) {
			$lines = explode(' ', str_replace(PHP_EOL, '', $buff));
			if($lines[0] == 'g') {
				array_shift($lines);
				$map_grp = array(
					'id'			=> 'pagid',
					'name'			=> 'name', 
					'start'			=> 'startip', 
					'end'			=> 'endip', 
					'ratein'		=> 'rate_in', 
					'rateout'		=> 'rate_out', 
					'maxonlinetime'	=> 'maxonlinetm', 
					'ifname'		=> 'ifname'
				);
				$ret = array();
/*
  ["id"]=>
  string(1) "2"
  ["name"]=>
  string(3) "AAA"
  ["start"]=>
  string(10) "10.10.10.2"
  ["end"]=>
  string(12) "10.10.10.255"
  ["ratein"]=>
  string(3) "222"
  ["rateout"]=>
  string(4) "1111"
  ["dns"]=>
  string(15) "114.114.114.114"
  ["maxonlinetime"]=>
  string(3) "500"
  ["clntepa"]=>
  string(4) "pass"
  ["ifname"]=>
  string(3) "le1"
*/
				foreach($lines as $volume)
				{
					$node = explode('=', $volume);
					if(count($node) < 2) 
						continue;
					
					if(isset($map_grp[$node[0]])) {
						if($node[0] == 'start' || $node[0] == 'end') 
							$ret[$map_grp[$node[0]]] =  ip2long($node[1]);
						else
						if($node[0] == 'name')
							$ret[$map_grp[$node[0]]] =  iconv('GB2312', 'UTF-8', $node[1]);
						else
							$ret[$map_grp[$node[0]]] =  $node[1];
					} 
					else 
					if($node[0] == 'dns') {
						$dns = explode(',', $node[1]);
						if(count($dns) > 0) $ret['dns0'] = ip2long($dns[0]);
						if(count($dns) > 1) $ret['dns1'] = ip2long($dns[1]);
					}
					else 
					if($node[0] == 'clntepa') {
						$clntepa = array('reject', 'login', 'pass');
						if(($key = array_search($node[1], $clntepa)) !== NULL)
							$ret[$node[0]] =  $key;
					}
				}
				$ret['ent_id'] = $ent_id;
				$ret['own_id'] = $user->own_id;
				if(\raas\apps\group\create($ret) !== false)
				$result['groupcnt']++;
				else
					array_push($result['error'], get_errmsg(''));
			}
			else
			if($lines[0] == 'u') {
				array_shift($lines);
				$map_grp = array(
					'n'			=> 'account', 
					'p'			=> 'pass',
					'b'			=> 'birth_time', 
					's'			=> 'start_time', 
					'e'			=> 'expire_time', 
					'ip'		=> 'bind_ipv4', 
					'mac'		=> 'bind_v4mac', 
					'maxonline'	=> 'maxcnt', 
					'outvlan'	=> 'bind_vlan',
				);
				$ret = array();
				$pagid = 0;
/*
  ["g"]=>
  string(1) "1"
  ["n"]=>
  string(2) "aa"
  ["p"]=>
  string(1) "1"
  ["b"]=>
  string(10) "1529645858"
  ["e"]=>
  string(10) "2019-06-22"
  ["s"]=>
  string(10) "2018-06-22"
  ["mac"]=>
  string(17) "10-00-00-00-00-10"
  ["maxonline"]=>
  string(2) "10"
  ["ip"]=>
  string(9) "11.0.0.10"
  ["outvlan"]=>
  string(2) "33"
  ["m"]=>
  string(24) "0;eee;wef;18301045608;VV"
*/
				foreach($lines as $volume)
				{
					$node = explode('=', $volume);
					if(count($node) < 2) 
						continue;
					
					if(isset($map_grp[$node[0]])) {
						if($node[0] == 's' || $node[0] == 'e') {
							$udate = new \DateTime($node[1]);
							$ret[$map_grp[$node[0]]] =  $udate->getTimestamp();
						}
						else
						if($node[0] == 'ip')
							$ret[$map_grp[$node[0]]] =  ip2long($node[1]);
						//if($node[0] == 'mac')
						//	$ret[$map_grp[$node[0]]] =  str_replace(':', '-', $node[1]);
						else
							$ret[$map_grp[$node[0]]] =  $node[1];
					} 
					else 
					if($node[0] == 'm') {
						$node[1] = iconv('GB2312', 'UTF-8', $node[1]);
						$m = explode(';', $node[1]);
						if(count($m) > 1) $ret['name'] =  $m[1];
						if(count($m) > 2) $ret['idcard'] =  $m[2];
						if(count($m) > 3) $ret['phone'] =  $m[3];
						if(count($m) > 4) $ret['remark'] =  $m[4];
					}
					else 
					if($node[0] == 'g') {
						$pagid = $node[1];
					}
				}
				$ret['ent_id'] = $ent_id;
				$ret['own_id'] = $user->own_id;
				if(($grp = _getpagid($pagid, $ent_id)) === false) continue;
				$ret['group_id'] = $grp->group_id;
				if(\raas\apps\account\create($ret) !== false)
				$result['acctcnt']++;
				else
					array_push($result['error'], get_errmsg(''));
			}
		}
	}
	fclose($fd);
	return $result;
}
