<?php
namespace raas\apps\user;

// _user_count_inc load
ni_app_load('counter', '_lib');
ni_app_load('entowner', '_lib');

function _create($frmData)
{
	global $nidb;


	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	try {
		if(insert_data('cloud_user', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$frmData['user_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(\raas\apps\entowner\_user_count_inc($frmData['own_id'], 1) === false) {
		return false;
	}
	if(\raas\apps\counter\_user_count_inc(COUNTER_RAAS_ID, 1) === false) {
		return false;
	}
	
	return $frmData;
}

function create($data)
{
	global $nidb;

	$filter = array(
		'parent_id'	=> new \FilterVarInt(),
		'own_id'	=> new \FilterVarID(true),
		'account'	=> new \FilterVarStr(true),
		'pass'		=> new \FilterVarStr(true),
		'disabled'	=> new \FilterVarInt(),
		'webmanage'	=> new \FilterVarInt(),
		'name'		=> new \FilterVarStr(),
		'email'		=> new \FilterVarEmail(),
		'idcard'	=> new \FilterVarStr(),
		'phone'		=> new \FilterVarStr(),
		'address'	=> new \FilterVarStr(),
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(empty($frmData['account'])) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不能为空！');
		return false;
	}
	
	// find account is exist
	$sql = "select `user_id` from cloud_user where `disabled` < " . TB_ROW_DEL_VALUE;
	$sql .=" and `account` = ?";
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $frmData['account'], \PDO::PARAM_STR);

		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_LAZY);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if($row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '“' . $frmData['name'] . '”帐号已存在');
		return false;
	}

	$nidb->beginTransaction();
	
	try {
		if(($frmData = _create($frmData)) === false) {
			$nidb->rollBack();
			return false;
		}
		$nidb->commit();
	} 
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	return $frmData;
}
