<?php
namespace raas\apps\user\group;


function lst_ent($data)
{
	// 获取所有企业与用户组列表
	global $nidb;

	if(isset($data['own_id']) == false
	|| ($own_id = intval($data['own_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的归属，请先登陆。');
		return false;
	}

	// 获取用户 token 
	$sql = "select gp.`group_id`, ent.`ent_id`, ent.`name` as `ent`, gp.`name` from cloud_group as `gp`";
	$sql.= " inner join cloud_enterpise as `ent`";
	$sql.= " on gp.`ent_id` = ent.`ent_id`";
	$sql.= " where ent.`disabled` < " . TB_ROW_DEL_VALUE;
	$sql.= " and gp.`disabled` < " . TB_ROW_DEL_VALUE;
	$sql.= " and gp.`own_id` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $own_id, \PDO::PARAM_INT);
		$sth->execute();

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($rows === false || count($rows) <= 0) 
		return (object)array();

	return $rows;
}

function get_option($data)
{
	global $nidb;


	$ids = array();
	if(is_string($data['ids'])) {
		$data['ids'] = explode(',', $data['ids']);
		foreach($data['ids'] as $key => $id) {
			if(empty($id) == false && ($id = intval($id)) > 0)
				array_push($ids, $id);
		}
	}
	else
	if(is_int($data['ids']) && $data['ids'] > 0) {
		array_push($ids, $data['ids']);
	}
	else {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，指定的用户组不正确！');
		return false;
	}
	if(count($ids) <= 0){
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，用户组为空！');
		return false;
	}

	if(isset($data['own_id']) == false
	|| ($own_id = intval($data['own_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的归属，请先登陆。');
		return false;
	}

	if(isset($data['user_id']) == false
	|| ($user_id = intval($data['user_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的操作员。');
		return false;
	}
	
	// 获取用户 token 
	$sql = "select gp.`group_id`, gp.`user_id`, item.`disabled` from cloud_user as `item`";
	$sql.= " inner join cloud_user_group_list as `gp`";
	$sql.= " on gp.`user_id` = item.`user_id`";
	$sql.= " where gp.`group_id` in (" . substr(str_repeat(',?', count($ids)), 1) . ")";
	$sql.= " and item.`own_id` = ? and item.`user_id` = ?";

	array_push($ids, $own_id);
	array_push($ids, $user_id);
	
	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($ids);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($rows === false || count($rows) <= 0) 
		return (object)array();

	return $rows;
}

function set_option($data)
{
	global $nidb;
	

	$ids = array();
	if(is_string($data['ids'])) {
		$data['ids'] = explode(',', $data['ids']);
		foreach($data['ids'] as $key => $id) {
			if(empty($id) == false && ($id = intval($id)) > 0)
				array_push($ids, $id);
		}
	}
	else
	if(is_int($data['ids']) && $data['ids'] > 0) {
		array_push($ids, $data['ids']);
	}
	else {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，指定的用户组不正确！');
		return false;
	}
	if(count($ids) <= 0){
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的请求，用户组为空！');
		return false;
	}
	
	if(isset($data['own_id']) == false
	|| ($own_id = intval($data['own_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的归属，请先登陆。');
		return false;
	}
	if(isset($data['user_id']) == false
	|| ($user_id = intval($data['user_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的操作员。');
		return false;
	}
	if(isset($data['checked']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '无效的操作，缺少参数：checked');
		return false;
	}
	$checked = intval($data['checked']);
	
	// 检查操作员是否存在 
	$sql = "select `user_id` from cloud_user";
	$sql.= " where `user_id` = ? and `own_id` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $user_id, \PDO::PARAM_INT);
		$sth->bindParam(2, $own_id, \PDO::PARAM_INT);
		$sth->execute();

		$row = $sth->fetch(\PDO::FETCH_LAZY);
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的操作员！');
		return false;
	}

	$entids = array();
	$grpids = array();
	// 获取组中对应的企业ID
	$sql = "select `ent_id`, `group_id` from cloud_group";
	$sql.= " where `own_id` = ? and `group_id` in (" . implode(',', $ids) . ")";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $own_id, \PDO::PARAM_INT);

		if($sth->execute()) {
			while ($row = $sth->fetch(\PDO::FETCH_ASSOC, \PDO::FETCH_ORI_NEXT)) {
				array_push($entids, $row['ent_id']);
				array_push($grpids, $row['group_id']);
			}
			$sth = null;
		}
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($grpids) <= 0 && count($entids) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的用户组不存在！');
		return false;
	}

	// 设置操作员关联企业
	$nidb->beginTransaction();
	try {
		if($checked) {
			// 选中
			$sql = "insert into `cloud_user_group_list` (`user_id`,`ent_id`,`group_id`) values (?,?,?) on duplicate key update `user_id` = ?, `ent_id` = ?, `group_id` = ?";

			foreach($grpids as $index => $id) {
				// 添加操作员关联的用户组
				$sth = $nidb->prepare($sql);
				$sth->bindParam(1, $user_id, \PDO::PARAM_INT);
				$sth->bindParam(2, $entids[$index], \PDO::PARAM_INT);
				$sth->bindParam(3, $grpids[$index], \PDO::PARAM_INT);
				$sth->bindParam(4, $user_id, \PDO::PARAM_INT);
				$sth->bindParam(5, $entids[$index], \PDO::PARAM_INT);
				$sth->bindParam(6, $grpids[$index], \PDO::PARAM_INT);
				$sth->execute();
			}
		}
		else {
			// 未选

			// 删除指定关联组
			$sql = "delete from cloud_user_group_list "; 
			$sql.= " where `group_id` in (" . implode(',', $ids) . ") and `user_id` = ?";

			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $user_id, \PDO::PARAM_INT);
			$sth->execute();
		}
	
		// 统计引用数
		// , count(*) as `cnt`
		$sql = "select `ent_id` from cloud_user_group_list where `user_id` = ? group by `ent_id`";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $user_id, \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
		if($rows)
			$count = count($rows);
		else
			$count = 0;

		// 更新计数器
		$sql = "update cloud_user set `entcnt` = ?  where `user_id` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $count, \PDO::PARAM_INT);
		$sth->bindParam(2, $user_id, \PDO::PARAM_INT);
		$sth->execute();

		$nidb->commit();
	} catch (\PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	return true;
}

