<?php
namespace raas\apps\user;

// 是否允许网页登陆；0：不允许；1：允许；2：企业管理员
define( 'WEB_TYPE_USER', 1);
define( 'WEB_TYPE_ROOT', 2);

// 支持库装载 | load lib
ni_library_load('nidb');
ni_library_load('session');
ni_app_load('role', 'library');
ni_app_load('entowner', 'info');

session_init();

// ********************************************************
// user session of manage
// ********************************************************
$GLOBALS['user'] = false;
$GLOBALS['entowner'] = false;
$GLOBALS['token'] = array();

function _webmanage_set($account, $val)
{
	global $nidb;

	$sql = "update cloud_user
			set `webmanage` = `webmanage` | ?
			where `account` = ?";

	try {
		$sth = $nidb->prepare($sql);
		
		$index = 1;
		$sth->bindParam($index++, $val, \PDO::PARAM_INT);
		$sth->bindParam($index++, $account, \PDO::PARAM_STR);
		
		$sth->execute();
		return $sth->rowCount();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
	}

	return false;
}

function _webmanage_del($account, int $val)
{
	global $nidb;

	$sql = "update cloud_user
			set `webmanage` = `webmanage` & ?
			where `account` = ?";
			
	$val = ~$val;
	try {
		$sth = $nidb->prepare($sql);
		
		$index = 1;
		$sth->bindParam($index++, $val, \PDO::PARAM_INT);
		$sth->bindParam($index++, $account, \PDO::PARAM_STR);
		
		$sth->execute();
		return $sth->rowCount();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
	}

	return false;
}

function logged()
{
	global $user, $entowner;

	if(isset($_SESSION['user_id']) == false) {
		set_errmsg(MSG_LEVEL_EXP, __function__, '请您先登陆！');
		return false;
	}
	
	if(($user = _infoid($_SESSION['user_id'])) === false) 
		return false;
	
	$entowner = \raas\apps\entowner\_info($user->own_id);
	
	return $user;
}

function _infoid(int $user_id)
{
	global $nidb;
	
	$sql = "select * from cloud_user where `user_id` = " . intval($user_id);
	
	try {
		$rows = $nidb->query($sql)->fetchAll(\PDO::FETCH_CLASS);
	} catch (PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) == 0) return false;

	return $rows[0];
}

function _getacct(string $account)
{
	global $nidb;
	
	$sql = "select * from cloud_user where `disabled` < " . TB_ROW_DEL_VALUE . " and `account` = ? limit 1";
	
	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $account, \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	} catch (PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) == 0) return false;

	return $rows[0];
}

function auth($data)
{
	global $nidb, $user;

	$filter = array(
		'account'	=> new \FilterVarStr(true, null, '__NULL__', '帐号不能为空！'),
		'pass'		=> new \FilterVarStr(true),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $errmsg);
		return false;
	}

	if($frmData['account'] == '') {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不能为空！');
		return false;
	}

	$frmData['account'] = strtolower($frmData['account']);

	if(($user = _getacct($frmData['account'])) === false) {
		// set_errmsg(MSG_LEVEL_DEF, __function__, '帐号不存在！');
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号或密码不正确！');
		return false;
	}

	if($user->pass != $frmData['pass']) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '帐号或密码不正确！');
		return false;
	}

	if($user->disabled) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '您的帐号已被停用！请联系客服。');
		return false;
	}

	$entowner = \raas\apps\entowner\_info($user->own_id);

	$_SESSION['account'] = $user->account;
	$_SESSION['user_id'] = $user->user_id;

	return $user;
}

function _auth_json_exit($appname, $appmethod, $func = '')
{
	global $user;

	// 验证用户 | auth user
	if(($user = logged()) == false) 
		json_error_exit(ERR_302, '请您先登陆！', 'index.php?r=user@login');

	// 验证身份 | auth token
	if(isallow($appname, $appmethod) === false)
		json_error_exit(ERR_FAILURE);

	// reset $_REQUEST
	$_REQUEST['own_id']	= $user->own_id;
	$_REQUEST['op_account'] = $user->account;
//	if(_hookrequest($func) === false) 
//		json_error_exit(ERR_FAILURE);

	return $user;
}

function _dump_login_page($msg)
{
	header("Content-type: text/html; charset=utf-8");
	echo "<script>location='index.php?r=user@login';</script>";
	exit;
}

function _auth_js_exit($appname, $appmethod, $func = '')
{
	global $user;

	// 验证用户 | auth user
	if(($user = logged()) == false)
		_dump_login_page(get_errmsg(''));

//	if($user->account == 'qcl' || $user->account == 'zhang.ruikang')
//		return $user;

	// 验证身份 | auth token
	if(isallow($appname, $appmethod) === false) {
//		set_errmsg(MSG_LEVEL_DEF, __function__, '您的操作权限不足！请联系管理员。');
		_dump_login_page(get_errmsg(''));
	}

	// reset $_REQUEST
	$_REQUEST['own_id']	= $user->own_id;
//	$_REQUEST['op_own_id']	= $user->own_id;
//	$_REQUEST['op_user_id']	= $user->user_id;
	$_REQUEST['op_account'] = $user->account;
//	if(_hookrequest($func) === false) {
//		header("Content-type: text/html; charset=utf-8");
//		echo get_errmsg('');
//		exit;
//	}

	return $user;
}

function auth_json_exit()
{

	// 检查调用堆栈
	$backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);

	if(count($backtrace) != 1)
		json_error_exit(ERR_FAILURE, '入口不正确！');

	$appdir = dirname($backtrace[0]["file"]);
	$appname = basename($appdir);
	$appmethod = basename($backtrace[0]["file"], '.php');

	if($appname == 'api') {
		$appname = basename(dirname($appdir));
		$ret = _auth_json_exit($appname, $appmethod);
	} else
		$ret = _auth_json_exit($appname, $appmethod);

	return $ret;
}

function auth_js_exit()
{

	// 检查调用堆栈
	$backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);

	$cnt = count($backtrace);
	if($cnt != 1) 
		_dump_login_page('入口不正确！');

	$appdir = dirname($backtrace[0]["file"]);
	$appname = basename($appdir);
	$appmethod = basename($backtrace[0]["file"], '.php');

	if($appname == 'api') {
		$appname = basename(dirname($appdir));
		$ret = _auth_js_exit($appname, $appmethod);
	} else
		$ret = _auth_js_exit($appname, $appmethod);

	return $ret;
}

function auth_exit($rettype = 'json')
{

	// 检查调用堆栈
	$backtrace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 2);

	$cnt = count($backtrace);
	if($cnt != 2) {
		if($rettype == 'json') 
			json_error_exit(ERR_FAILURE, '入口不正确！');
		else
			_dump_login_page('入口不正确！');
	}
	
	$appdir = dirname($backtrace[1]["file"]);
	$appname = basename($appdir);
	$appmethod = basename($backtrace[1]["file"], '.php');

	if($rettype == 'json') {
		if($appname == 'api')
			$ret = _auth_json_exit(dirname($appdir), $appmethod);
		else
			$ret = _auth_json_exit($appname, $appmethod);
	}
	else {
		if($appname == 'api')
			$ret = _auth_js_exit(dirname($appdir), $appmethod);
		else
			$ret = _auth_js_exit($appname, $appmethod);
	}

	return $ret;
}

function logout()
{
	global $user;

	session_clean();

	$user = false;
}

function isallow($appname, $method)
{
	global $user, $token;

	if(isset($token[$appname]) === false) {
		$tk = \raas\apps\role\get_token($appname);
		if($tk === false) return false;

		//	权限bit位值，最大支持映射32种权限，
		//	目前为8种：
		//	1：info、2：list、4：search、8：disable、
		//	16：enable、32：save、64：create、128：remove、256：menu

		$token[$appname] = $tk;
	}
	else
		$tk = $token[$appname];

	

	if($tk->system && is_superman($user->account) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '操作权限不足！请联系管理员。');
		return false;
	}

	if(isset($tk->tokenmap->$method) == false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无效的权限！请联系管理员。');
		return false;
	}

	if(($tk->curval & $tk->tokenmap->$method->val) == 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '您的权限不足！请联系管理员。');
		return false;
	}

	return true;
}

function _hookrequest($func = '')
{
	global $user;

	if(empty($func)) {
//		$_REQUEST['user_id']	= $user->user_id;
//		$_REQUEST['group_id']	= $user->group_id;
		$_REQUEST['own_id']		= $user->own_id;
		return true;
	}
	else {
		if(function_exists($func) == false) {
			set_errmsg(MSG_LEVEL_LOAD, __function__, $func . ' function not exist');
			return false;
		}
	}

	return call_user_func($func, $user);
}

